#include "NMaskAlgorithm.h"
#include "AnalysisAlgorithmMap.h"
#include "SctData/NMaskTestResult.h"
#include "SctData/RawScanResult.h"
#include "SctData/ModuleDefect.h"
#include "SctData/ModuleElement.h"
#include <TH1.h>
#include <TH2.h>

using namespace std;
using namespace boost;
using namespace SctData;
using namespace Sct;

namespace SctAnalysis {

bool NMaskAlgorithm::inMap = AnalysisAlgorithmMap::instance().setAlgorithm("NMaskTest", auto_ptr<AnalysisAlgorithm>(new NMaskAlgorithm()));

shared_ptr<AnalysisAlgorithm> NMaskAlgorithm::clone(const TestData& testData, const string& moduleName) const throw() {
    return shared_ptr<AnalysisAlgorithm>(new NMaskAlgorithm(testData, moduleName));
}

void NMaskAlgorithm::canAddRawScanResult(const string& name) throw(Sct::LogicError, Sct::IoError) {

    shared_ptr<NMaskTestResult> result =  dynamic_pointer_cast<NMaskTestResult> ( getTestResult() );

    if (result.get()==0) {
        result = shared_ptr<NMaskTestResult> (new NMaskTestResult());
        setTestResult( result );
        initializeTestResult();
    }

    addRaw(name);

    if ( !result->hasAllRaws() )
        return;

    shared_ptr<const RawScanResult> raw = result->getRaw(0);
    TH2D& link0 = raw->getScanData(0);
    TH2D& link1 = raw->getScanData(1);
    int nTriggers = (int)result->getTestPointAt(0);
    
    for (unsigned int channel = 0; channel<Sct::nChannelModule; ++channel) {
	for (unsigned int bin = 0; bin<link0.GetNbinsY(); ++bin) {
	    int content = (int) channel < Sct::nChannelLink ? link0.GetBinContent(channel+1, bin+1) : link1.GetBinContent(channel+1-nChannelLink, bin+1);
	    if (channel%nChannelChip < bin) {
		if (content != 0) {
		    result->getDefects().addDefect(ModuleDefect::OVER, Channel(channel));
		    break;
		}
	    } else {
		if (content != nTriggers) {
		    result->getDefects().addDefect(ModuleDefect::UNDER, Channel(channel));
		    break;
		}
	    }
	}
    }
    if (result->getDefects().defectAffectingElement(Module()))
	result->setPassed(false);
    else 
	result->setPassed(true);
        
    finish();
}
} // end of namespace SctAnalysis
