#ifndef STROBEDELAYALGORITHM_H
#define STROBEDELAYALGORITHM_H
#include "AnalysisAlgorithm.h"

namespace SctData {
    class StrobeDelayTestResult;
    class FitScanResult;
}

namespace SctAnalysis {
    /**
       Algorithm for single-scan Strobe-Delay Test.
       Uses a single FitScanResult to find the optimum value of the strobe delay.
    */
class StrobeDelayAlgorithm : public AnalysisAlgorithm {
public:        
    /**
       over-ride
     */
    virtual void canAddFitScanResult(const string& name) throw(Sct::LogicError, Sct::IoError);
    /**
     * over-ride
     */
    virtual boost::shared_ptr<AnalysisAlgorithm> clone(const TestData& testData, const string& moduleName) const throw();
    /** 
	constructor 
     */
    StrobeDelayAlgorithm(const TestData& testData, const string& moduleName) throw() : AnalysisAlgorithm(testData, moduleName) {}
    /** 
	destructor 
     */
    virtual ~StrobeDelayAlgorithm() throw() {}
private:
    StrobeDelayAlgorithm() {}
    static bool inMap;
    /** 
	finds the optimum - a fractions of the way from the mid-rise to the mid-fall of the 
	function.
     */
    static void optimize(SctData::StrobeDelayTestResult& result, const SctData::FitScanResult& fitted) throw(Sct::LogicError);
};
}
#endif //#ifndef STROBEDELAYALGORITHM_H
