#ifndef SCTCALIBRATIONCONTROLLER_NMASKTEST_H
#define SCTCALIBRATIONCONTROLLER_NMASKTEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/Sct_CalibrationController.hh"

#include "../SctApiException.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {
    
class NMaskTest : public TestRequestImpl {
public:	
    static NMaskTest& instance() {
	static NMaskTest rc;
	return rc;
    }
    
    virtual void startTest(Sct_SctApi_T_SctApi& api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = &api;
	
	Sct_SctApiStatus status;
	
	api.modifyABCDVar(&status, ST_COMPRESSION, 1);     // Compression mode 1
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "NMaskTest::startTest", __FILE__, __LINE__);
	
	api.modifyABCDVar(&status, ST_EDGE_DETECT, 0);     // Edge detect off
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "NMaskTest::startTest", __FILE__, __LINE__);
	
	api.modifyABCDVar(&status, ST_SEND_MASK, 1);    // Send mask on
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "NMaskTest::startTest", __FILE__, __LINE__);
	
	name = "NMaskTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	
	testPoints.push_back(nTriggers);
    }
    
    virtual Sct_SctApi_T_Scan* getNextScan(unsigned long index) {
	ScanLibrary& sl = CalibrationControllerImpl::instance().getScanLibrary();
	return &sl.nmaskScan(nTriggers);
    }
    
    virtual bool canFeedback() {
	return false;
    }
    
    virtual void endTest() {
	api = 0;
    }
    
private:
    NMaskTest() {}
    Sct_SctApi_T_SctApi* api;    
    static const int nTriggers = 100;
};
}
}
#endif //SCTCALIBRATIONCONTROLLER_NMASKTEST_H
