#ifndef PIPELINETEST_H
#define PIPELINETEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/Sct_CalibrationController.hh"
#include "../CalibrationControllerImpl.h"

#include "../SctApiException.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {
    
class PipelineTest : public TestRequestImpl {
public:	
    static PipelineTest& instance() {
	static PipelineTest rc;
	return rc;
    }
    
    virtual void startTest(Sct_SctApi_T_SctApi& inapi, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = &inapi;
	
	Sct_SctApiStatus status;
	api->modifyABCDVar(&status, ST_COMPRESSION, 1);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "PipelineTest::startTest", __FILE__, __LINE__);
	
	api->modifyABCDVar(&status, ST_EDGE_DETECT, 0);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "PipelineTest::startTest", __FILE__, __LINE__);
	
	api->modifyABCDVar(&status, ST_SEND_MASK, 0);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "PipelineTest::startTest", __FILE__, __LINE__);

	name = "PipelineTest";
	nScans = 2;
	testPoints.push_back(0);
	testPoints.push_back(1);
	testVariable=ST_MASK;
    }
    
    virtual Sct_SctApi_T_Scan* getNextScan(unsigned long index) {
	if (api == 0) 
	    throw InvariantViolatedError("Api pointer is 0 in getNextScan!", __FILE__, __LINE__);
	
	// Set configuration
	Sct_SctApiStatus status;
	api->modifyABCDVar(&status, ST_MASK, 3.0*(1-index) );
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "PipelineTest::getNextScan", __FILE__, __LINE__);
	
	// Create scan
        Sct_SctApi_T_Scan* s = api->defaultScan(&status);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "PipelineTest::getNextScan", __FILE__, __LINE__);
	
        s->setNTrigs(&status, 100);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "PipelineTest::getNextScan", __FILE__, __LINE__);
	
	// Adjust trigger
        Sct_SctApi_T_Trigger* t = s->getTrigger1(&status);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "PipelineTest::getNextScan", __FILE__, __LINE__);

	// 129 is critical to the pulse arriving correctly
        t->softPulseL1A(&status, 140, 129);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "PipelineTest::getNextScan", __FILE__, __LINE__);

	// "Command" 1 is the delay (starting at 140)
	t->setCommIncr(&status, 1, 1);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "PipelineTest::getNextScan", __FILE__, __LINE__);

	// Configure scan
	s->configure(&status, ST_ELAPSED, 140, 151, 1);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "PipelineTest::getNextScan", __FILE__, __LINE__);
	
	return s;
    }
    
    virtual bool canFeedback() {
	return false;
    }
    
    virtual void endTest() {
	if (api == 0) 
	    throw InvariantViolatedError("Api pointer is 0 in endTest!", __FILE__, __LINE__);
	
	Sct_SctApiStatus status;
	api->modifyABCDVar(&status, ST_MASK, 0);	//Not that this unsets any previously set mask.
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "PipelineTest::getNextScan", __FILE__, __LINE__);

	
	api = 0;
    }

private:
    PipelineTest() {}
    Sct_SctApi_T_SctApi* api;
};
}
}
#endif //PIPELINETEST_H
