#ifndef SCT_FILE_OSTREAMFILE_H
#define SCT_FILE_OSTREAMFILE_H

#include "../OStream.h"
#include <fstream>
#include <string>
#include <cstring>

using std::fstream;
using std::string;
using std::strcpy;

namespace Sct {
namespace File {
    
class OStreamFile : public virtual OStream {
public:
    OStreamFile(const string& fileName) : file(fileName.c_str()) {
	if (!file.is_open()) {
	    throw FileError("Couldn't open file", fileName);
	}
    }
    
    virtual ~OStreamFile() {
	if (file.is_open()) {
	    file.close();
	}
    }
    
    virtual OStream & operator<<(bool val) {
	file << val << " ";
	return *this;
    }
    
    virtual OStream & operator<<(char val) {
	file << (int)val << " ";
	return *this;
    }
    
    virtual OStream & operator<<(unsigned char val) {
	file << (unsigned int)val << " ";
	//cout << "WriteUChar: " << (int)val << " at: " << file.tellp() << endl;
	return *this;
    }
    
    virtual OStream & operator<<(short val) {
	file << val << " ";
	return *this;
    }
    
    virtual OStream & operator<<(unsigned short val){
	file << val << " ";
	return *this;
    }
    
    virtual OStream & operator<<(int val) {
	file << val << " ";
	return *this;
    }
    
    virtual OStream & operator<<(unsigned int val) {
	file << val << " ";
	return *this;
    }
    
    virtual OStream & operator<<(long val) {
	file << val << " ";
	return *this;
    }
    
    virtual OStream & operator<<(unsigned long val) {
	file << val << " ";
	return *this;
    }
    
    virtual OStream & operator<<(double val) {
	file << val << " ";
	return *this;
    }
    
    virtual OStream & operator<<(float val) {
	file << val << " ";
	return *this;
    }
    
    virtual OStream & operator<<(const std::string & val) {
	file << val << " ";
	return *this;
    }
    
//    virtual OStream & operator<<(OWLDate & val)  = 0;
//    virtual OStream & operator<<(OWLTime & val)  = 0;

    virtual OStream & operator<<(const char * str) {
	file << str << " ";
	return *this;
    }
    
    virtual OStream & operator<<(const unsigned char * str) {
	file << str << " ";
	return *this;
    }
    
    virtual OStream & put(const bool* p, size_t size) {
	file << size << " ";
	for (unsigned int i=0; i<size; ++i) {
	    file << p[i] << " ";
	}
	return *this;
    }
    
    virtual OStream & put(const char* p, size_t size) {
	file << size << " ";
	for (unsigned int i=0; i<size; ++i) {
	    file << (int)p[i] << " ";
	}
	return *this;
    }
    
    virtual OStream & put(const unsigned char* p, size_t size) {
	file << size << " ";
	for (unsigned int i=0; i<size; ++i) {
	    file << (int)p[i] << " ";
	}
	return *this;
    }
  
    virtual OStream & put(const short* p, size_t size) {
	file << size << " ";
	for (unsigned int i=0; i<size; ++i) {
	    file << p[i] << " ";
	}
	return *this;
    }
  
    virtual OStream & put(const unsigned short* p, size_t size) {
	file << size << " ";
	for (unsigned int i=0; i<size; ++i) {
	    file << p[i] << " ";
	}
	return *this;
    }
  
    virtual OStream & put(const int* p, size_t size) {
	file << size << " ";
	for (unsigned int i=0; i<size; ++i) {
	    file << p[i] << " ";
	}
	return *this;
    }
  
    virtual OStream & put(const unsigned int* p, size_t size) {
	file << size << " ";
	for (unsigned int i=0; i<size; ++i) {
	    file << p[i] << " ";
	}
	return *this;
    }
  
    virtual OStream & put(const long* p, size_t size) {
	file << size << " ";
	for (unsigned int i=0; i<size; ++i) {
	    file << p[i] << " ";
	}
	return *this;
    }
  
    virtual OStream & put(const unsigned long* p, size_t size) {
	file << size << " ";
	for (unsigned int i=0; i<size; ++i) {
	    file << p[i] << " ";
	}
	return *this;
    }
  
    virtual OStream & put(const float* p, size_t size) {
	file << size << " ";
	for (unsigned int i=0; i<size; ++i) {
	    file << p[i] << " ";
	}
	return *this;
    }
  
    virtual OStream & put(const double* p, size_t size) {
	file << size << " ";
	for (unsigned int i=0; i<size; ++i) {
	    file << p[i] << " ";
	}
	return *this;
    }
  
    virtual OStream & put(const std::string* p, size_t size) {
	file << size << " ";
	for (unsigned int i=0; i<size; ++i) {
	    file << p[i] << " ";
	}
	return *this;
    }
  
//    virtual OStream & put(const OWLDate* p, size_t size) = 0;
//    virtual OStream & put(const OWLTime* p, size_t size) = 0;
    
private:
    ofstream file;
};
}
}
#endif //SCT_FILE_OSTREAMFILE_H
