#include "FullBypassTestResultStreamer.h"

using namespace Sct;

namespace SctData {
namespace IO {

FullBypassTestResultStreamer::FullBypassTestResultStreamer() throw() {}

bool FullBypassTestResultStreamer::inMap = IOManager::addToMap("SctData::FullBypassTestResult", auto_ptr<Streamer>(new FullBypassTestResultStreamer()));

shared_ptr<Streamable> FullBypassTestResultStreamer::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError){
    boost::shared_ptr<Streamable> ad (new FullBypassTestResult());
    read(in, *ad, manager);
    return ad;
}

void FullBypassTestResultStreamer::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {

    //Let superclass do its work
    TestResultStreamer::write(out, ob, manager);
    
    const FullBypassTestResult& mytest = dynamic_cast<const FullBypassTestResult&>(ob);

    out << mytest.getVddNom() << mytest.getNConfig();

    for (unsigned int ichip=0; ichip<nChipModule; ichip++) {
        writeChipResult(out, mytest.getChipResult(ichip), manager);
    }
}

void FullBypassTestResultStreamer::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    TestResultStreamer::read(in, ob, manager);
    
    FullBypassTestResult& mytest = dynamic_cast<FullBypassTestResult&>(ob);

    bool mybool; float myfloat; unsigned myunsigned;
    in >> myfloat; mytest.setVddNom(myfloat);
    in >> myunsigned; mytest.setNConfig(myunsigned);

    for (unsigned int ichip=0; ichip<nChipModule; ichip++) {
        readChipResult(in, mytest.getChipResult(ichip), manager);
    }
}
    
void FullBypassTestResultStreamer::writeChipResult(OStream& out, const FullBypassTestResult::ChipFBResult& c, const IOManager& manager) const throw (LogicError, IoError) {
    out << c.status_direct << c.vdd_direct << c.status_bypass << c.vdd_bypass;
}

void FullBypassTestResultStreamer::readChipResult(IStream& in, FullBypassTestResult::ChipFBResult& c, const IOManager& manager) const throw(LogicError, IoError) {
    in >> c.status_direct >> c.vdd_direct >> c.status_bypass >> c.vdd_bypass;
}

} 
}
