#include "ModuleDefectListStreamer.h"
#include "../ModuleDefectList.h"
#include "../ModuleDefect.h"
#include "Sct/LogicErrors.h"
#include "Sct/IoExceptions.h"

using namespace Sct;

namespace SctData {
namespace IO {

ModuleDefectListStreamer::ModuleDefectListStreamer() throw() {}

bool ModuleDefectListStreamer::inMap = IOManager::addToMap("SctData::ModuleDefectList", auto_ptr<Streamer>(new ModuleDefectListStreamer()));

void ModuleDefectListStreamer::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    //cast should always be safe
    const ModuleDefectList& m = dynamic_cast<const ModuleDefectList&>(ob);

    const vector< shared_ptr<const ModuleDefect> >& list = m.getAllDefects();
    out << list.size();
    for (unsigned int i=0; i<list.size(); ++i) {
        ModuleElement el=list[i]->getModuleElement();
        out << helper.getRepresentation(*(list[i])) << el.getFirst() << el.getNChannels();
    }
}

shared_ptr<Streamable> ModuleDefectListStreamer::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
    shared_ptr<Streamable> m (new ModuleDefectList());
    read(in, *m, manager);
    return m;
}

void ModuleDefectListStreamer::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    ModuleDefectList& m = dynamic_cast<ModuleDefectList&>(ob);

    int nDefects = -1;
    int rep = 1;
    unsigned int startChannel = 0;
    unsigned int nChannels = 0;

    in >> nDefects;
    for (int i=0; i<nDefects; ++i) {
        in >> rep >> startChannel >> nChannels;
        m.addDefect(helper.getFromRep(rep), ModuleElement(startChannel, startChannel+nChannels-1));
    }
}
}
}
