#ifndef SCTDATA_IO_RAWSCANRESULTSTREAMER_H
#define SCTDATA_IO_RAWSCANRESULTSTREAMER_H

#include "../RawScanResultIOHelper.h"
#include "ScanResultStreamer.h"

class SctData::RawScanResult;
class TH1;

using namespace Sct;

namespace SctData {
namespace IO {


/**
This class is an ISSerializer for reading/writing RawScanResults to/from IS.
*/
class RawScanResultStreamer : public ScanResultStreamer {
public:
    //Streamer overrides
    virtual void write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError);
    virtual shared_ptr<Streamable> read(IStream& in, const IOManager& manager) const throw(LogicError, IoError);
    virtual void read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError);   

private:
    /**
    Based on the data type, passes control onto the appropriate function for reading out the data
    */
    void readData(IStream& in, RawScanResult& raw) const throw(LogicError, IoError);

    /**
    Reads in data in Slice format from IS.
    */
    void readSliceData(unsigned int size, unsigned short width, IStream& in, RawScanResult& raw) const throw(LogicError, IoError);

    /**
    Reads in data in ROOT format from IS.  I.e. same data format as is published by this.
    */
    void readRootData(unsigned int size, unsigned short width, IStream& in, RawScanResult& raw) const throw(LogicError, IoError);
    
    //Data
    mutable RawScanResultIOHelper helper;		///< The IOHelper
    RawScanResultStreamer() throw();			///< Constructor is private
    static bool inMap;					///< The static dummy variable for the IOManager map
};

}
}
#endif //#ifndef SCTDATA_IO_RAWSCANRESULTSTREAMER_H
