#include "TimeWalkTestResultStreamer.h"

using namespace Sct;

namespace SctData {
    namespace IO {
	
	TimeWalkTestResultStreamer::TimeWalkTestResultStreamer() throw() {}
	
bool TimeWalkTestResultStreamer::inMap = IOManager::addToMap("SctData::TimeWalkTestResult", auto_ptr<Streamer>(new TimeWalkTestResultStreamer()));
	
	shared_ptr<Streamable> TimeWalkTestResultStreamer::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError){
	    boost::shared_ptr<Streamable> ad (new TimeWalkTestResult());
	    read(in, *ad, manager);
	    return ad;
	}
	
	void TimeWalkTestResultStreamer::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {

	    cout << "TimeWalkTestResultStreamer calling TestResultStreamer::write()" << endl;
	    //Let superclass do its work
	    TestResultStreamer::write(out, ob, manager);
	    
	    cout << "TimeWalkTestResultStreamer doing dynamic_cast" << endl;
	    const TimeWalkTestResult& mytest = dynamic_cast<const TimeWalkTestResult&>(ob);
	    
	    for (unsigned int ichip=0; ichip<nChipModule; ichip++) {
		writeChipResult(out, mytest.getChipResult(ichip), manager);
	    }
	}
	
	void TimeWalkTestResultStreamer::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
	    TestResultStreamer::read(in, ob, manager);
	    
	    TimeWalkTestResult& mytest = dynamic_cast<TimeWalkTestResult&>(ob);
	    
	    for (unsigned int ichip=0; ichip<nChipModule; ichip++) {
		readChipResult(in, mytest.getChipResult(ichip), manager);
	    }
	}
	
	void TimeWalkTestResultStreamer::writeChipResult(OStream& out, const TimeWalkTestResult::ChipTWResult& c, const IOManager& manager) const throw (LogicError, IoError) {
	    out << c.timewalk << c.calibration;
	}
	
	void TimeWalkTestResultStreamer::readChipResult(IStream& in, TimeWalkTestResult::ChipTWResult& c, const IOManager& manager) const throw(LogicError, IoError) {
	    in >> c.timewalk >> c.calibration;
	}
	
    } 
}
