#include "StrobeDelayTestResult.h"
#include "StrobeDelayVariable.h"
#include "NullVariable.h"
#include "FitObject.h"
#include "FitScanResult.h"
#include "ConfigurationVariable.h"
#include "TopHatFitObject.h"

#include "Sct/SctParameters.h"
#include "Sct/LogicErrors.h"
#include "Sct/OutOfRangeError.h"

//ROOT includes
#include <TF1.h>

using namespace Sct;

namespace SctData {

StrobeDelayTestResult::StrobeDelayTestResult() throw() : m_optima(nChipModule){
    setFraction();
}

StrobeDelayTestResult::StrobeDelayTestResult(const unsigned int runNumber, const string& moduleName) throw() 
    : TestResult(runNumber, moduleName, NullVariable::instance(), StrobeDelayVariable::instance()), 
      m_optima(nChipModule){
    setFraction();
}

string StrobeDelayTestResult::getClassName() const  throw() {
    return "SctData::StrobeDelayTestResult";
}

void StrobeDelayTestResult::addScan(unsigned int scanNumber, double testPoint) throw (LogicError) {
    if (data.size() != 0) throw InvariantViolatedError("StrobeDelayTestResult::addScan - can only contain 1 scan", __FILE__, __LINE__);
    TestResult::addScan(scanNumber, testPoint);
}

double StrobeDelayTestResult::getOptimum(unsigned i) const throw(LogicError) {
    if (i>=nChipModule)
        throw OutOfRangeError<unsigned>("StrobeDelayTestResult::getOptimum", __FILE__, __LINE__, i,0,nChipModule-1);
    return m_optima[i];
}

    void StrobeDelayTestResult::setOptimum(unsigned int chip, double optimum) throw(Sct::LogicError) {
	if (chip>=nChipModule)
	    throw OutOfRangeError<unsigned>("StrobeDelayTestResult::setOptimum", __FILE__, __LINE__,chip,0,nChipModule-1);
	m_optima[chip] = optimum;    
}
}
