#ifndef STROBEDELAYTESTRESULT_H
#define STROBEDELAYTESTRESULT_H

#include "FitObject.h"
#include "TestResult.h"

class TF1;
class TGraph;

using namespace std;

namespace SctData {
	
  /**
   * A derived class of TestResult, containing the results data for a Strobe Delay Test.
   * Such a test requires only a single scan.<P>
   * The result of the analysis is an optimum delay value for each of the 12 chips.
   * @author Alan Barr
   */
class StrobeDelayTestResult : public TestResult {
public:
    StrobeDelayTestResult() throw();

    ~StrobeDelayTestResult() throw() {}
    
    /**
    Constructor.  
    */
    StrobeDelayTestResult(const unsigned int runNumber, const string& moduleName) throw() ;
      
    //override
    virtual string getClassName() const throw() ;
  
    /**
      Override addScan to ensure there is only 1 scanNumber
      */
    virtual void addScan(unsigned int scanNumber, double testPoint = 0) throw (LogicError);    
    
    /**
     * set the position parameter for the optimum efficiency.
     */
    void setFraction(const double f=0.25) throw() {m_fraction=f;}
    
    /**
     * get the position parameter for the optimum efficiency.
     */
    double getFraction() const throw() {return m_fraction;}
        
    /// get the number of optmimised chip delays (usually 0 or 12).
    unsigned int getNOptima() const throw() {return m_optima.size();}
    
    void setNOptima(unsigned int n) throw() {m_optima.resize(n);}
    
    /// get the value of one of the chip delays (chip `i').
    double getOptimum(unsigned i) const throw(Sct::LogicError);
    
    /**
      set the optimum of one chip to a particular value
      @param chip in range 0->11
      @parram optimum the value of the optimum to set
    */
    void setOptimum(unsigned int chip, double optimum) throw(Sct::LogicError);
    
private:
    double m_fraction; 			///< the fraction of the distance from rise to fall for optimum efficiency.
    vector<double> m_optima; 		///< the optimum value of the delay.
};
  
} // end of namespace SctData

#endif //#ifndef STROBEDELAYTESTRESULT_H
