// File: TApi.cxx

#include <iostream>

#include "TApi.h"

#include "SctApiImpl.h"

ClassImp(TApi)
ClassImp(TScanDef)
ClassImp(TTrigger)

// TApi *tapi;

// Constructor
TApi::TApi(int initDebug){
  worker = new SctApi::SctApi(initDebug);
}

// Destructor
/*
  The destructor.
*/
TApi::~TApi(){
  delete worker;
}

// ROD Diagnostics

void TApi::initialiseAll(int scanNumber) {
  worker->initialiseAll(scanNumber);
}

void TApi::shutdownAll() {
  worker->shutdownAll();
}

void TApi::setRunNumber(int newRun) {
  worker->setRunNumber(newRun);
}

void TApi::setScanNumber(int newScan) {
  worker->setScanNumber(newScan);
}

void TApi::flashLED(unsigned int partition, unsigned int crate, unsigned int rod,
                    unsigned long slaveNumber, long period, long flashes) {
  worker->flashLED(partition, crate, rod, slaveNumber, period, flashes);
}

void TApi::echo(unsigned int partition, unsigned int crate, unsigned int rod,
                unsigned int length, long *data) {
  worker->echo(partition, crate, rod, length, data);
}

void TApi::awaitResponse(unsigned int partition, unsigned int crate, unsigned int rod, int timeout) {
  worker->awaitResponse(partition, crate, rod, timeout);
}

unsigned long *TApi::getResponse(unsigned int partition, unsigned int crate, unsigned int rod) {
  unsigned long length;
  unsigned long *result = worker->getResponse(partition, crate, rod, length);
  if(length < 2) result = 0;
  else {
    if(result[0] != length) {
      cout << "Result length mismatch!\n";
    }
  }
  return result;
}

unsigned long *TApi::getResponse(unsigned int partition, unsigned int crate, unsigned int rod, 
                                 unsigned long *length) {
  unsigned long myLength;
  unsigned long *result = worker->getResponse(partition, crate, rod, myLength);
  *length = myLength;
  return result;
}

// Get a message from a ROD text buffer
bool TApi::getRodMessage(unsigned int partition, unsigned int crate, unsigned int rod, 
			 char *buffer, int &length) {
  return worker->getRodMessage(partition, crate, rod, buffer, length);
}

UINT32 TApi::findModule(string sn) {
  return worker->findModule(sn);
}

UINT32 TApi::findModule(INT32 mur, INT32 module) {
  return worker->findModule(mur, module);
}

void TApi::sendABCDModule(UINT32 mid, UINT32 bank, UINT32 type) {
  worker->sendABCDModule(mid, bank, type);
}

void TApi::getABCDModule(UINT32 bank) {
  worker->getABCDModule(bank);
}

void TApi::getABCDModule(UINT32 mid, UINT32 bank) {
  worker->getABCDModule(mid, bank);
}

void TApi::modifyABCDMask(UINT32 mid, UINT32* mask) {
  worker->modifyABCDMask(mid, mask);
}

void TApi::modifyABCDTrims(UINT32 mid, UINT8* trims) {
  worker->modifyABCDTrims(mid, trims);
}

void TApi::modifyABCDVar(UINT32 typ, FLOAT32 var) {
  worker->modifyABCDVar(typ, var);
}

void TApi::modifyABCDVar(UINT32 mid, UINT32 typ, FLOAT32 var) {
  worker->modifyABCDVar(mid, typ, var);
}

void TApi::modifyABCDVar(UINT32 mid, UINT32 c, UINT32 typ, FLOAT32 var) {
  worker->modifyABCDVar(mid, c, typ, var);
}

void TApi::modifyBOCParam(unsigned int partition, unsigned int crate, unsigned int rod,
                            unsigned int channel, unsigned int type, unsigned int val) {
  worker->modifyBOCParam(partition, crate, rod, channel, type, val);
}

void TApi::loadConfiguration() {
  worker->loadConfiguration();
}

void TApi::configureBOC(unsigned int partition, unsigned int crate, unsigned int rod) {
  worker->configureBOC(partition, crate, rod);
}

UINT32 TApi::findBarrelModule(INT32 barrel, INT32 row, INT32 number) {
  return worker->findBarrelModule(barrel, row, number);
}

UINT32 TApi::findEndcapModule(INT32 disk, INT32 ring, INT32 number) {
  return worker->findEndcapModule(disk, ring, number);
}

void TApi::addDebugPrimList(unsigned long length, long index, long id, long version,
			    unsigned long * body) {
  worker->addDebugPrimList(length, index, id, version, body);
}

void TApi::sendDebugPrimList(unsigned int partition, unsigned int crate, unsigned int rod) {
  worker->sendDebugPrimList(partition, crate, rod);
}

void TApi::sendDebugSlavePrimList(unsigned int partition, unsigned int crate, unsigned int rod,
                                  unsigned int slave, bool await, bool response) {
  worker->sendDebugSlavePrimList(partition, crate, rod, slave, await, response);
}

void TApi::createDebugPrimList() {
  worker->createDebugPrimList();
}


/*
  Dump contents of a block of DSP memory to cout.
*/
int TApi::dspBlockDump(unsigned int partition, unsigned int crate, unsigned int rod,
                       long dspStart, long numWords, long dspNumber){
  return worker->dspBlockDump(partition, crate, rod, dspStart, numWords, dspNumber);
}

int TApi::dspBlockDumpFile(unsigned int partition, unsigned int crate, unsigned int rod,
                           long dspStart, long numWords, long dspNumber, const char *filename)
{
  return worker->dspBlockDumpFile(partition, crate, rod, dspStart, numWords, dspNumber, filename);
}

/*
  Read contents of a block of DSP memory.
*/
unsigned long *TApi::dspBlockRead(unsigned int partition, unsigned int crate, unsigned int rod,
				  long dspStart, long numWords, long dspNumber, long *length){
  long thing;
  unsigned long *result = worker->dspBlockRead(partition, crate, rod, dspStart, numWords, dspNumber, thing);
  *length = thing;
  return result;
}

int TApi::dspBlockWrite(unsigned int partition, unsigned int crate, unsigned int rod,
                        unsigned long *buffer, unsigned long dspAddress, long numWords, long dspNumber) {
  return worker->dspBlockWrite(partition, crate, rod, buffer, dspAddress, numWords, dspNumber);
}

unsigned long TApi::dspSingleRead(unsigned int partition, unsigned int crate, unsigned int rod,
				  const unsigned long dspAddr, long dspNumber) {
  return worker->dspSingleRead(partition, crate, rod, dspAddr, dspNumber);
}

/// Write a single 32b word to MasterDSP SDRAM via HPI
void TApi::dspSingleWrite(unsigned int partition, unsigned int crate, unsigned int rod,
                   unsigned long dspAddr, unsigned long value, long dspNumber) {
  worker->dspSingleWrite(partition, crate, rod, dspAddr, value, dspNumber);
}

void TApi::defaultScan(int type) {
  worker->defaultScan(type);
}

void TApi::tidyHistogramming() {
  worker->tidyHistogramming();
}


void TApi::doScan(TScanDef tScan) {
  SctApi::Scan *scan = tScan.getScan();

  if(scan)
    worker->doScan(*scan);
  else 
    std::cout << "Bad scan at Root interface level\n";
}

void TApi::doRawScan(TScanDef tScan, int delay, bool configure, bool clkBy2) {
  SctApi::Scan *scan = tScan.getScan();

  if(scan)
    worker->doRawScan(*scan, delay, clkBy2);
  else 
    std::cout << "Bad scan at Root interface level\n";
}

void TApi::sendTrigger(unsigned int partition, unsigned int crate, unsigned int rod, TTrigger tTrig) {
  const SctApi::Trigger *trig = tTrig.getTrigger();

  if(trig)
    worker->sendTrigger(partition, crate, rod, trig);
  else 
    std::cout << "Bad trigger at Root interface level\n";
}

void TApi::status() {
  worker->status();
}

unsigned long TApi::readRodCommandReg(unsigned int partition, unsigned int crate, unsigned int rod,
				      long regNumber) {
  return worker->readRodCommandReg(partition, crate, rod, regNumber);
}

unsigned long TApi::readRodStatusReg(unsigned int partition, unsigned int crate, unsigned int rod,
				     long regNumber) {
  return worker->readRodStatusReg(partition, crate, rod, regNumber);
}

void TApi::setABCDModule(UINT32 mid, UINT32 bank) {
  worker->setABCDModule(mid, bank);
}

unsigned long *TApi::retrieveModule(UINT32 mid) {
  return (unsigned long*)worker->retrieveModule(mid);
}

void TApi::printABCDModule(int mid) {
  worker->printABCDModule(mid);
}

void TApi::printABCDRodModule(int mid, int bank) {
  worker->printABCDRodModule(mid, bank);
}

void TApi::printBOCSetup(unsigned int partition, unsigned int crate, unsigned int rod) {
  worker->printBOCSetup(partition, crate, rod);
}

void TApi::printBOCRegisters(unsigned int partition, unsigned int crate, unsigned int rod) {
  worker->printBOCRegisters(partition, crate, rod);
}

void TApi::rawData(unsigned int partition, unsigned int crate, unsigned int rod, 
		   int delay, int units, bool setMask, TTrigger tTrig) {
  const SctApi::Trigger *trig = tTrig.getTrigger();

  if(trig)
    worker->rawData(partition, crate, rod, delay, units, setMask, trig);
  else 
    std::cout << "Bad trigger to raw data at Root interface level\n";
}

char *TApi::probe(unsigned int partition, unsigned int crate, unsigned int rod, signed int harness) {
  vector<char> result = worker->probe(partition, crate, rod, harness);
  int length = result.size();
  char *buffer = new char[length + 1];

  for (int i=0; i<length; i++) {
    buffer[i] = result[i];
  }

  buffer[length] = 0;

  return buffer;
}

char *TApi::probeWithTrigger(unsigned int partition, unsigned int crate, unsigned int rod, 
			     TTrigger tTrig, signed int harness) {
  const SctApi::Trigger *trig = tTrig.getTrigger();

  if(trig) {
    vector<char> result = worker->probeWithTrigger(partition, crate, rod, trig, harness);
    int length = result.size();
    char *buffer = new char[length + 1];

    for (int i=0; i<length; i++) {
      buffer[i] = result[i];
    }

    buffer[length] = 0;

    return buffer;
  } else {
    std::cout << "Bad trigger to raw data at Root interface level\n";
    return 0;
  }
}

void TApi::rodMode(unsigned int partition, unsigned int crate, unsigned int rod,
                   int mode, int flag, int fifoSetup, int nBins, int delay, int message) {
  worker->rodMode(partition, crate, rod, 
                  mode, flag, fifoSetup, nBins, delay, message);
}

void TApi::bocHistogram(unsigned int partition, unsigned int crate, unsigned int rod, 
                        unsigned int samples, unsigned int numLoops)
{
  worker->bocHistogram(partition, crate, rod, samples, numLoops);
}

void TApi::setDebugLevel(unsigned int level) {
  worker->setDebugLevel(level);
}

unsigned int TApi::getDebugLevel() {
  return worker->getDebugLevel();
}

void TApi::debugStepHistogram() 
{
  worker->debugStepHistogram();
}

void TApi::debugContinueHistogram()
{
  worker->debugContinueHistogram();
}

void TApi::standardRegisterDump(unsigned int partition, unsigned int crate, unsigned int rod) 
{
  worker->standardRegisterDump(partition, crate, rod);
}

void TApi::lasersOff() {
  worker->lasersOff();
}

void TApi::decodeEvent(unsigned int partition, unsigned int crate, unsigned int rod, 
                       int sl, int index, bool extFlag, bool errorType) {
  worker->decodeEvent(partition, crate, rod, sl, index, extFlag, errorType);
}

void TApi::freeTriggers(unsigned int partition, unsigned int crate,
			double freq) {
  worker->freeTriggers(partition, crate, freq);
}

void TApi::stopTriggers(unsigned int partition, unsigned int crate) {
  worker->stopTriggers(partition, crate);
}

void TApi::timL1A(unsigned int partition, unsigned int crate) {
  worker->stopTriggers(partition, crate);
}

void TApi::timCalL1A(unsigned int partition, unsigned int crate, int delay) {
  worker->stopTriggers(partition, crate);
}

void TApi::requestHardReset(UINT32 mid) {
  worker->requestHardReset(mid);
}

void TApi::printHex(UINT32 num, int width) {
  worker->printHex(num, width);
}


TScanDef::TScanDef() :
  trigsPerBurst(200),
  scanVariable(1),  scanVariable2(1),
  format(1), full(0), bits32(1), loopCalLine(0),
  distSlave(0), debug(0),
  worker(new SctApi::ScanDefImpl)
{
  // A large threshold scan
  configure(1, 0.0, 400.0, 5.0);

  // With a single L1A trigger
  trigSequence.singleL1A();

  // So the worker has pointers to the TriggerImpl's in the trigsequences
  update();
}

TScanDef::~TScanDef() {
  delete worker;
}

TScanDef::TScanDef(const TScanDef &other) :
  trigsPerBurst(other.trigsPerBurst),

  scanVariable(other.scanVariable),
  scanVariable2(other.scanVariable2),

  trigSequence(other.trigSequence),
  trigSequence2(other.trigSequence2),

  format(other.format),
  full(other.full), 
  bits32(other.bits32),
  loopCalLine(other.loopCalLine),

  distSlave(other.distSlave),
  debug(other.debug),

  worker(new SctApi::ScanDefImpl(*other.worker))
{
}

void TScanDef::update() {
  // Only copy the things which are defined as parameters
  // (ScanPoints are set up in worker by the configure method)
  worker->setNTrigs(trigsPerBurst);

  worker->setScanVariable1(scanVariable);
  worker->setScanVariable2(scanVariable2);

  // Copy triggers
  worker->setTrigger1(trigSequence.getTrigger());
  worker->setTrigger2(trigSequence2.getTrigger());

  // Set options
  worker->setOption(SctApi::Scan::FORMAT, format);
  worker->setOption(SctApi::Scan::FULL, full);
  worker->setOption(SctApi::Scan::BITS32, bits32);
  worker->setOption(SctApi::Scan::LOOPCALLINE, loopCalLine);
  worker->setOption(SctApi::Scan::DISTSLAVE, distSlave);
  worker->setOption(SctApi::Scan::DEBUG, debug);
}

void TScanDef::print() {
  update();
  worker->print(); // copyScan(this).print();
}

void TScanDef::configure(UINT16 type, FLOAT32 start, FLOAT32 stop, FLOAT32 step) {
  scanVariable = type;
  worker->configure(type, start, stop, step);
}

void TScanDef::configure2(UINT16 type, FLOAT32 start, FLOAT32 stop, FLOAT32 step) {
  scanVariable2 = type;
  worker->configure2(type, start, stop, step);
}

// This is the default for both set 0 and set 1
TTrigger::TTrigger() {
  worker = new SctApi::TriggerImpl;

  incCmd = 0;
  incData = 0;

  update();
}

TTrigger::TTrigger(const TTrigger &other) {
  worker = new SctApi::TriggerImpl(*other.worker);

  incCmd = other.incCmd;
  incData = other.incData;
}

TTrigger::~TTrigger() {
  // This is deleted by ScanDefImpl not via TScanDef
  //  delete worker;
}

void TTrigger::singleL1A() {
  worker->singleL1A();
}

void TTrigger::doubleL1A(int delay) {
  worker->doubleL1A(delay);
}

void TTrigger::calL1A(int delay) {
  worker->calL1A(delay);
}

void TTrigger::pulseL1A(int delay) {
  worker->pulseL1A(delay);
}

void TTrigger::softL1A(int delay) {
  worker->softL1A(delay);
}

void TTrigger::softCalL1A(int delay, int delay2) {
  worker->softCalL1A(delay, delay2);
}

void TTrigger::softPulseL1A(int delay, int delay2) {
  worker->softPulseL1A(delay, delay2);
}

void TTrigger::bcL1A(int delay) {
  worker->bcL1A(delay);
}

void TTrigger::bcCalL1A(int delay, int delay2) {
  worker->bcCalL1A(delay, delay2);
}

void TTrigger::bcPulseL1A(int delay, int delay2) {
  worker->bcPulseL1A(delay, delay2);
}

void TTrigger::update() {
  worker->setCommIncr(incCmd, incData);
}

SctApi::Trigger *TTrigger::getTrigger() {
  update();

  return worker;
}

// Copy TScanDef scan to implementation of Scan interface 
SctApi::Scan *TScanDef::getScan() {
//   if((tScan.trigSequence.trigCommand.GetSize() != tScan.trigSequence.trigData.GetSize()) ||
//      (tScan.trigSequence2.trigCommand.GetSize() != tScan.trigSequence2.trigData.GetSize()) ||
//      ((tScan.trigSequence.trigCommand.GetSize() != tScan.trigSequence2.trigCommand.GetSize()) && 
//       (tScan.trigSequence2.trigData.GetSize() != 0))) {
//     cout << "Trigger arrays not compatible! Aborting scan...\n";
//     return;
//   }

  update();

  return worker;
}
