package SctApi.gui;

import Sct_SctApi.*;
import ipc.*;

import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.JScrollPane;

import org.omg.CORBA.IntHolder;

public class ScanGui extends JFrame {
    private Sct_SctApi.SctApi api;

    final private ScanGui finalGui;

    public static void main(String args[]) {
        System.out.println("Hello world");

        JFrame frame = new ScanGui();

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });  

        frame.pack();
        frame.setVisible(true);
    }

    public ScanGui() {
        super("SCTAPI Scan GUI");

        finalGui = this;

        Partition p= new Partition("SCT");
        try {
            org.omg.CORBA.Object obj = p.lookup("SCTSctApiServer", "Test_Server"); 
            if (obj == null) {
                System.out.println("Object SCTSctApiServer not found in server Test_Server");
            } else {
                api = Sct_SctApi.SctApiHelper.narrow(obj);
            }
        } catch (org.omg.CORBA.BAD_PARAM bp) {
            System.err.println("Couldn't get api object: " + bp);
        } catch (org.omg.CORBA.COMM_FAILURE cf) {
            System.err.println("CORBA communications error: " + cf);
        } catch (java.lang.Exception e) {
            System.err.println("Other error" + e);
        }

	if (api==null) {
	    System.err.println("Error (check SCTAPI running)");
            System.exit(0);
	}
        System.out.println("Found SCTAPI");

        JPanel buttons = new JPanel();

        JButton close = new JButton("Close");
        close.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    dispose();
                }
            });
        buttons.add(close);

        JButton shutdown = new JButton("Shutdown server");
        shutdown.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    System.out.println("Shutdown system");
                    api.shutdownAll();
                    System.out.println("Done");
                    api.destroy();
                    dispose();
                    System.exit(0);
                }
            });
        buttons.add(shutdown);

        JButton probeButton = new JButton("Probe Modules");
        probeButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JFrame probeFrame = new ProbeView(api);

                    probeFrame.addWindowListener(new WindowAdapter() {
                            public void windowClosing(WindowEvent e) {
                                e.getWindow().dispose();
                            }
                        });

                    probeFrame.pack();
                    probeFrame.setVisible(true);
                }
            });
        buttons.add(probeButton);

        JButton newScan = new JButton("New scan");
        newScan.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    Sct_SctApi.Scan scan = api.defaultScan();

                    scan.configure((short)1, 0.0, 100.0, 10.0);

                    scan.setNTrigs(1000);

                    scan.getTrigger1().singleL1A();

                    JFrame frame = new ScanView(api, scan);

                    frame.addWindowListener(new WindowAdapter() {
                            public void windowClosing(WindowEvent e) {
                                e.getWindow().dispose();
                            }
                        });
                    frame.pack();
                    frame.setVisible(true);
                }
            });
        buttons.add(newScan);

        JButton listScan = new JButton("List scans");
        listScan.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    Sct_SctApi.Scan[] scans = api.knownScans();

                    JFrame scanListFrame = new ScanList(api, scans);

                    scanListFrame.addWindowListener(new WindowAdapter() {
                            public void windowClosing(WindowEvent e) {
                                e.getWindow().dispose();
                            }
                        });

                    scanListFrame.pack();
                    scanListFrame.setVisible(true);
                }
            });
        buttons.add(listScan);

        JButton statusButton = new JButton("Show status");
        statusButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    api.status();
                }
            });
        buttons.add(statusButton);

        getContentPane().add(buttons, BorderLayout.SOUTH);

        System.out.println("Set up GUI, wait for it to run");
    }
}
