#include "ServiceTests.h"
#include "archiving/ArchRawScanResult.h"
#include "archiving/ArchFitScanResult.h"
#include "archiving/ArchNPtGainTestResult.h"
#include "archiving/ArchNoiseOccupancyTestResult.h"
#include "archiving/ArchStrobeDelayTestResult.h"
#include "archiving/ArchTrimRangeTestResult.h"
#include "archiving/ArchTimeWalkTestResult.h"
#include "archiving/ArchPipelineTestResult.h"
#include "archiving/ArchFullBypassTestResult.h"
#include "archiving/ArchNMaskTestResult.h"


#include "Sct/SctNames.h"
#include "is/isinfo.h"
#include "Sct/IS/IOManagerIS.h"
#include "SctData/TestResult.h"
#include "SctData/RawScanResult.h"
#include "SctData/FitScanResult.h"

#include <string>
//#include "Sct/shared_ptr_utility.h"

using namespace Sct;
using namespace SctData;
using namespace Sct::IS;
using namespace boost;

namespace SctArchivingServiceTests {

ArchivingServiceTests::ArchivingServiceTests() {}


void ArchivingServiceTests::runTests() {
    cout<<" I am in ArchivingServiceTests::runTests() " <<endl;
    IPCPartition& p = SctNames::getPartition();
    ISInfoReceiver r(p);
    
    r.subscribe( "TestData", ".*TestResult.*", MyISCallbackTests,this);
    r.run();
   
}




void ArchivingServiceTests::MyISCallbackTests(ISCallbackInfo * isc) {
    cout<<" I am in ArchivingService::MyISCallbackTests " <<endl;
    ArchivingServiceTests* a = static_cast<ArchivingServiceTests*>(isc->parameter());
    cout<< " !!!! IS Object Name = " << isc->type().name()<<endl;
    a->archiveTests(isc);
}




void ArchivingServiceTests::archiveTests(ISCallbackInfo * isc) {
    cout<<" I am in ArchivingServiceTests::archiveTests " <<endl;
    
    if (isc->reason() != ISInfoCreated && isc->reason() != ISInfoUpdated ) return;

    
    shared_ptr<Serializable> ob = IOManagerIS::instance().read(*isc);
    cout << "!!!!!!!!! CLASS NAME ==== " << ob->getClassName() <<endl;
    if (ob == 0) {
        cerr << "Failed to get object from IS" << endl;
        return;
    }

    
    if(ob->getClassName() == "SctData::NoiseOccupancyTestResult") {
      cout<<" The Test Obj is NoiseOccupancyTestResult "<<endl;
      //NoiseOccupancyTestResult* test = dynamic_cast<NoiseOccupancyTestResult*>(ob);
      shared_ptr<NoiseOccupancyTestResult> result = boost::dynamic_pointer_cast<NoiseOccupancyTestResult>(ob);

      ArchNoiseOccupancyTestResult archnotest(*result);
      archnotest.SetUNIXDir();
      archnotest.SaveNoiseOccupancy();
      cout<< "Noise Occupancy Test Correctly Saved " << endl;

    }
    else if(ob->getClassName() == "SctData::NPtGainTestResult") {
      cout<<" The Test Obj is NPtGainTestResult "<<endl;
      shared_ptr<NPtGainTestResult> result = boost::dynamic_pointer_cast<NPtGainTestResult>(ob);
      ArchNPtGainTestResult archNPtGtest(*result);
      archNPtGtest.SetUNIXDir();
      archNPtGtest.SaveNPtGain();
      cout<< "NPt Gain Test Correctly Saved " << endl;

    }
    else if(ob->getClassName() == "SctData::StrobeDelayTestResult") {
      cout<<" The Test Obj is StrobeDelayTestResult "<<endl;
      shared_ptr<StrobeDelayTestResult> result = boost::dynamic_pointer_cast<StrobeDelayTestResult>(ob);
      ArchStrobeDelayTestResult archSDtest(*result);
      archSDtest.SetUNIXDir();
      archSDtest.SaveStrobeDelay();
      cout<< "Strobe Delay Test Correctly Saved " << endl;

    }
    else if(ob->getClassName() == "SctData::TrimRangeTestResult") {
      cout<<" The Test Obj is TrimRangeTestResult "<<endl;
      shared_ptr<TrimRangeTestResult> result = boost::dynamic_pointer_cast<TrimRangeTestResult>(ob);
      ArchTrimRangeTestResult archTRtest(*result);
      archTRtest.SetUNIXDir();
      archTRtest.SaveTrimRange();
      cout<< "Trim Range Test Correctly Saved " << endl;

    }
    else if(ob->getClassName() == "SctData::TimeWalkTestResult") {
      cout<<" The Test Obj is TimeWalkTestResult "<<endl;
      shared_ptr<TimeWalkTestResult> result = boost::dynamic_pointer_cast<TimeWalkTestResult>(ob);
      ArchTimeWalkTestResult archTWtest(*result);
      archTWtest.SetUNIXDir();
      archTWtest.SaveTimeWalk();
      cout<< "TimeWalk Test Correctly Saved " << endl;

    }
    else if(ob->getClassName() == "SctData::FullBypassTestResult") {
      cout<<" The Test Obj is FullBypassTestResult "<<endl;
      shared_ptr<FullBypassTestResult> result = boost::dynamic_pointer_cast<FullBypassTestResult>(ob);
      ArchFullBypassTestResult archFBtest(*result);
      archFBtest.SetUNIXDir();
      archFBtest.SaveFullBypass();
      cout<< "FullBypass Test Correctly Saved " << endl;

    }
    else if(ob->getClassName() == "SctData::PipelineTestResult") {
      cout<<" The Test Obj is PipelineTestResult "<<endl;
      shared_ptr<PipelineTestResult> result = boost::dynamic_pointer_cast<PipelineTestResult>(ob);
      ArchPipelineTestResult archPtest(*result);
      archPtest.SetUNIXDir();
      archPtest.SavePipeline();
      cout<< "Pipeline Test Correctly Saved " << endl;

    }
    else if(ob->getClassName() == "SctData::NMaskTestResult") {
      cout<<" The Test Obj is NMaskTestResult "<<endl;
      shared_ptr<NMaskTestResult> result = boost::dynamic_pointer_cast<NMaskTestResult>(ob);
      ArchNMaskTestResult archNMtest(*result);
      archNMtest.SetUNIXDir();
      archNMtest.SaveNMask();
      cout<< "NMask Test Correctly Saved " << endl;

    }
    else {
      cout<< "ERROR: the object on IS is an unknown Test type" <<endl;
    }
    
    
    cout << "Bye bye" <<endl;
    

   
}

}
