#include <memory>

#include "configipc.h"
#include "marshalling.h"
#include "Sct/ILUMarshalling.h"

using namespace std;
using namespace Sct;
using namespace SctConfiguration;

void myThrow(sctConfStatus *_status);

ConfigIPC::ConfigIPC()
{
  IPCPartition p("SCT");
  handle = p.lookup(sctConf_C_Configuration_instanceName,     // "Test_Server", 
                    sctConf_C_Configuration_serverName,       // "SCTConfigurationServer", 
                    sctConf_T_Configuration::ILUClassRecord);
  if (handle == 0 ) {
    std::cout << "Error" << std::endl;
    throw ConfigurationException("Can't initialise! (Can't find IPC config)");
  }
}

ConfigIPC::~ConfigIPC()
{
}

std::list<unsigned int> ConfigIPC::listPartitions()
{
  sctConfStatus status;	
  status.returnCode = 0;
  sctConf_T_PartitionList myList = handle->listPartitions(&status);
  myThrow(&status);

  return copyILUToList<_sctConf_T_PartitionList_sequence, unsigned int>(myList);
}

std::list<unsigned int> ConfigIPC::listCratesInPartition(unsigned int partition)
{
  sctConfStatus status;	
  status.returnCode = 0;
  sctConf_T_CrateList myList = handle->listCratesInPartition(&status, partition);
  myThrow(&status);

  return copyILUToList<_sctConf_T_CrateList_sequence, unsigned int>(myList);
}

std::list<unsigned int> ConfigIPC::listRodsInCrate(unsigned int partition, unsigned int crate)
{
  sctConfStatus status;	
  status.returnCode = 0;
  sctConf_T_RodList myList = handle->listRodsInCrate(&status, partition, crate);
  myThrow(&status);

  return copyILUToList<_sctConf_T_RodList_sequence, unsigned int>(myList);
}

std::list<unsigned int> ConfigIPC::listMURSInRod(unsigned int partition, unsigned int crate, unsigned int rod)
{
  sctConfStatus status;	
  status.returnCode = 0;
  sctConf_T_MURList myList = handle->listMURSInRod(&status, partition, crate, rod);
  myThrow(&status);

  return copyILUToList<_sctConf_T_MURList_sequence, unsigned int>(myList);
}

std::list<std::string> ConfigIPC::listModulesInMUR(unsigned int partition, unsigned int MUR)
{
  sctConfStatus status;	
  status.returnCode = 0;
  sctConf_T_ModuleList myList = handle->listModulesInMUR(&status, partition, MUR);
  myThrow(&status);

  return copyILUToList<_sctConf_T_ModuleList_sequence, string>(myList);
}

std::list<std::string> ConfigIPC::listRModulesInMUR(unsigned int partition, unsigned int MUR)
{
  sctConfStatus status;	
  status.returnCode = 0;
  sctConf_T_ModuleList myList = handle->listRModulesInMUR(&status, partition, MUR);
  myThrow(&status);

  return copyILUToList<_sctConf_T_ModuleList_sequence, string>(myList);
}

std::list<std::string> ConfigIPC::listAllModules()
{
  sctConfStatus status;	
  status.returnCode = 0;
  sctConf_T_ModuleList myList = handle->listAllModules(&status);
  myThrow(&status);

  return copyILUToList<_sctConf_T_ModuleList_sequence, string>(myList);
}

std::list<std::string> ConfigIPC::listUnusedModules()
{
  sctConfStatus status;	
  status.returnCode = 0;
  sctConf_T_ModuleList myList = handle->listUnusedModules(&status);
  myThrow(&status);

  return copyILUToList<_sctConf_T_ModuleList_sequence, string>(myList);
}

bool ConfigIPC::isDummyCrate(unsigned int partition, unsigned int crate)
{
  sctConfStatus status;	
  status.returnCode = 0;
  bool ret = handle->isDummyCrate(&status, partition, crate);
  myThrow(&status);

  return ret;
}


ABCDModule ConfigIPC::getModuleConfig(const std::string module)
{
  sctConfStatus status;	
  status.returnCode = 0;
  auto_ptr<sctConf_T_ABCDModule> iluModule (handle->getModuleConfig(&status, const_cast<char *>(module.c_str())));
  myThrow(&status);

  return copyILUToABCDModule(*iluModule);
}

RodConfig ConfigIPC::getRodConfig(unsigned int partition, unsigned int crate, unsigned int rod)
{
  sctConfStatus status;	
  status.returnCode = 0;
  sctConf_T_RodConfig *config = handle->getRodConfig(&status, partition, crate, rod);
  myThrow(&status);

  return copyILUToRodConfig(config);
}

BOCChannelConfig ConfigIPC::getBOCConfig(unsigned int partition, unsigned int crate, unsigned int rod, 
                                         unsigned int channel)
{
  sctConfStatus status;	
  status.returnCode = 0;
  sctConf_T_BOCChannelConfig *config = handle->getBOCConfig(&status, partition, crate, rod, channel);
  myThrow(&status);

  return copyILUToBOCChannelConfig(config);
}

BOCGlobalConfig ConfigIPC::getBOCGlobalConfig(unsigned int partition, unsigned int crate, unsigned int rod)
{
  sctConfStatus status;	
  status.returnCode = 0;
  sctConf_T_BOCGlobalConfig *config = handle->getBOCGlobalConfig(&status, partition, crate, rod);
  myThrow(&status);

  return copyILUToBOCGlobalConfig(config);
}

char *ConfigIPC::getFibreMappings(unsigned int partition, unsigned int crate, unsigned int rod)
{
  sctConfStatus status;	
  status.returnCode = 0;
  char *array = new char[48*3];
  copyILUToArray<sctConf_T_CharBlock, char>(handle->getFibreMappings(&status, partition, crate, rod), array, 48*3);

  myThrow(&status);

  return array;
}

TimConfig ConfigIPC::getTimConfig(unsigned int partition, unsigned int crate)
{
  sctConfStatus status;	
  status.returnCode = 0;
  sctConf_T_TimConfig *config = handle->getTimConfig(&status, partition, crate);
  myThrow(&status);

  return copyILUToTimConfig(config);
}

void ConfigIPC::setFibreMapping(unsigned int partition, unsigned int crate, unsigned int rod, 
                                unsigned int channel, unsigned int tx, unsigned int rx0, unsigned int rx1)
{
  sctConfStatus status;	
  status.returnCode = 0;
  handle->setFibreMapping(&status, partition, crate, rod, channel, tx, rx0, rx1);
  myThrow(&status);
}

void ConfigIPC::printModuleConfig(const ABCDModule &conf)
{
  sctConfStatus status;	
  status.returnCode = 0;

  auto_ptr<sctConf_T_ABCDModule> config (copyABCDModuleToILU(conf));

  handle->printModuleConfig(&status, config.get());
  myThrow(&status);
}

MURType ConfigIPC::getMURType(unsigned int MUR)
{
  sctConfStatus status;	
  status.returnCode = 0;

  MURType result;

  result = (MURType)handle->getMURType(&status, MUR);
  myThrow(&status);

  return result;
}

void ConfigIPC::translateToROD(unsigned int MUR, unsigned int module,
                               unsigned int &partition, unsigned int &crate, 
                               unsigned int &rod, unsigned int &channel)
{
  sctConfStatus status;	
  status.returnCode = 0;

  unsigned long p, c, r, ch;
  handle->translateToROD(&status, MUR, module, &p, &c, &r, &ch);
  partition = p; crate = c; rod = r; channel = ch;
  myThrow(&status);
}

void ConfigIPC::translateToRROD(unsigned int MUR, unsigned int module,
                                unsigned int &partition, unsigned int &crate, 
                                unsigned int &rod, unsigned int &channel)
{
  sctConfStatus status;	
  status.returnCode = 0;

  unsigned long p, c, r, ch;
  handle->translateToRROD(&status, MUR, module, &p, &c, &r, &ch);
  partition = p; crate = c; rod = r; channel = ch;
  myThrow(&status);
}

void ConfigIPC::translateToSN(unsigned int MUR, unsigned int module,
                              std::string &sn)
{
  sctConfStatus status;	
  status.returnCode = 0;

  char *inputString = 0;

  handle->translateToSN(&status, MUR, module, &inputString);
  myThrow(&status);

  sn = inputString;
}

void ConfigIPC::translateToBarrel(unsigned int MUR, unsigned int module,
                                  unsigned int &barrel, unsigned int &row, int &number)
{
  sctConfStatus status;	
  status.returnCode = 0;

  unsigned long b, r;
  long n;
  handle->translateToBarrel(&status, MUR, module, &b, &r, &n);
  barrel = b; row = r; number = n;
  myThrow(&status);
}

void ConfigIPC::translateToEndcap(unsigned int MUR, unsigned int module,
                                  unsigned int &disk, unsigned int &ring, unsigned int &number)
{
  sctConfStatus status;	
  status.returnCode = 0;

  unsigned long d, r, n;
  handle->translateToEndcap(&status, MUR, module, &d, &r, &n);
  disk = d; ring = r; number = n;
  myThrow(&status);
}

void ConfigIPC::translateToPowerSupply(unsigned int MUR, unsigned int module,
                                       unsigned int &partition, 
                                       unsigned int &crate, unsigned int &channel)
{
  sctConfStatus status;
  status.returnCode = 0;

  unsigned long p, c, ch;
  handle->translateToPowerSupply(&status, MUR, module, &p, &c, &ch);
  partition = p; crate = c; channel = ch;
  myThrow(&status);
}

void ConfigIPC::translateFromROD(unsigned int partition, unsigned int crate, 
                                 unsigned int rod, unsigned int channel,
                                 unsigned int &MUR, unsigned int &module)
{
  sctConfStatus status;
  status.returnCode = 0;

  unsigned long m, mo;
  handle->translateFromROD(&status, partition, crate, rod, channel, &m, &mo);
  MUR = m; module = mo;
  myThrow(&status);
}

void ConfigIPC::translateFromRROD(unsigned int partition, unsigned int crate,
                                  unsigned int rod, unsigned int channel,
                                  unsigned int &MUR, unsigned int &module)
{
  sctConfStatus status;
  status.returnCode = 0;

  unsigned long m, mo;
  handle->translateFromRROD(&status, partition, crate, rod, channel, &m, &mo);
  MUR = m; module = mo;
  myThrow(&status);
}

void ConfigIPC::translateFromSN(const std::string sn,
                                unsigned int &MUR, unsigned int &module)
{
  sctConfStatus status;
  status.returnCode = 0;

  unsigned long m, mo;
  handle->translateFromSN(&status, const_cast<char *>(sn.c_str()), &m, &mo);
  MUR = m; module = mo;
  myThrow(&status);
}

void ConfigIPC::translateFromBarrel(unsigned int barrel, unsigned int row, int number,
                                    unsigned int &MUR, unsigned int &module)
{
  sctConfStatus status;
  status.returnCode = 0;

  unsigned long m, mo;
  handle->translateFromBarrel(&status, barrel, row, number, &m, &mo);
  MUR = m; module = mo;
  myThrow(&status);
}

void ConfigIPC::translateFromEndcap(unsigned int disk, unsigned int ring, unsigned int number,
                                    unsigned int &MUR, unsigned int &module)
{
  sctConfStatus status;
  status.returnCode = 0;

  unsigned long m, mo;
  handle->translateFromEndcap(&status, disk, ring, number, &m, &mo);
  MUR = m; module = mo;
  myThrow(&status);
}

void ConfigIPC::translateFromPowerSupply(unsigned int partition, 
                                         unsigned int crate, unsigned int channel, 
                                         unsigned int &MUR, unsigned int &module)
{
  sctConfStatus status;
  status.returnCode = 0;

  unsigned long m, mo;
  handle->translateFromPowerSupply(&status, partition, crate, channel, &m, &mo);
  MUR = m; module = mo;
  myThrow(&status);
}

void ConfigIPC::configureModuleFromStructure(const std::string module, const ABCDModule conf)
{
  sctConfStatus status;	
  status.returnCode = 0;

  auto_ptr<sctConf_T_ABCDModule> config (copyABCDModuleToILU(conf));

  handle->configureModuleFromStructure(&status, const_cast<char *>(module.c_str()), config.get());
  myThrow(&status);
}

void ConfigIPC::configureModuleFromFile(const std::string filename)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->configureModuleFromFile(&status, const_cast<char *>(filename.c_str()));
  myThrow(&status);
}

void ConfigIPC::configureROD(unsigned int partition, unsigned int crate, unsigned int rod, RodConfig conf)
{
  sctConfStatus status;	
  status.returnCode = 0;

  sctConf_T_RodConfig *config = copyRodConfigToILU(conf);

  handle->configureROD(&status, partition, crate, rod, config);
  myThrow(&status);
}

void ConfigIPC::configureTIM(unsigned int partition, unsigned int crate, TimConfig conf)
{
  sctConfStatus status;	
  status.returnCode = 0;

  sctConf_T_TimConfig *config = copyTimConfigToILU(conf);

  handle->configureTIM(&status, partition, crate, config);
  myThrow(&status);
}

void ConfigIPC::configureBOC(unsigned int partition, unsigned int crate, unsigned int rod, const BOCGlobalConfig &conf)
{
  sctConfStatus status;	
  status.returnCode = 0;

  sctConf_T_BOCGlobalConfig *config = copyBOCGlobalConfigToILU(conf);

  handle->configureBOC(&status, partition, crate, rod, config);
  myThrow(&status);
}

void ConfigIPC::clearAll()
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->clearAll(&status);
  myThrow(&status);
}

void ConfigIPC::namePartition(int partition, const std::string name)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->namePartition(&status, partition, const_cast<char *>(name.c_str()));
  myThrow(&status);
}

void ConfigIPC::configureBOCChannel(int MUR, int position, const BOCChannelConfig &conf)
{
  sctConfStatus status;	
  status.returnCode = 0;

  sctConf_T_BOCChannelConfig *config = copyBOCChannelConfigToILU(conf);

  handle->configureBOCChannel(&status, MUR, position, config);
  myThrow(&status);
}

void ConfigIPC::mapModuleMUR(int MUR, int order, int RMUR, int rorder, std::string number)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->mapModuleMUR(&status, MUR, order, RMUR, rorder, const_cast<char *>(number.c_str()));
  myThrow(&status);
}

void ConfigIPC::mapRODMUR(int partition, int crate, int rod, int order, int number)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->mapRODMUR(&status, partition, crate, rod, order, number);
  myThrow(&status);
}

void ConfigIPC::mapBarrelMUR(int MUR, int barrel, int row, int position)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->mapBarrelMUR(&status, MUR, barrel, row, position);
  myThrow(&status);
}

void ConfigIPC::mapEndcapMUR(int MUR, int disk, int quadrant, int position)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->mapEndcapMUR(&status, MUR, disk, quadrant, position);
  myThrow(&status);
}

void ConfigIPC::swapMURNames(int MUR1, int MUR2)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->swapMURNames(&status, MUR1, MUR2);
  myThrow(&status);
}

void ConfigIPC::mapPowerChannel(int MUR, int number, unsigned int partition, unsigned int crate, unsigned int channel)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->mapPowerChannel(&status, MUR, number, partition, crate, channel);
  myThrow(&status);
}

void ConfigIPC::modifyPowerParam(int MUR, int number, string name, float value)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->modifyPowerParam(&status, MUR, number, const_cast<char*>(name.c_str()), value);
  myThrow(&status);
}

void ConfigIPC::modifyDefaultPowerParam(string name, float value)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->modifyDefaultPowerParam(&status, const_cast<char *>(name.c_str()), value);
  myThrow(&status);
}
 
float ConfigIPC::getPowerParam(int MUR, int number, string name)
{
  sctConfStatus status;	
  status.returnCode = 0;

  float result = handle->getPowerParam(&status, MUR, number, const_cast<char *>(name.c_str()));
  myThrow(&status);

  return result;
}

void ConfigIPC::loadConfiguration(const std::string &filename)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->loadConfiguration(&status, const_cast<char *>(filename.c_str()));
  myThrow(&status);
}

void ConfigIPC::saveConfiguration(const std::string filename)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->saveConfiguration(&status, const_cast<char *>(filename.c_str()));
  myThrow(&status);
}

void ConfigIPC::saveModuleConfiguration(const std::string modsn, const std::string filename)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->saveModuleConfiguration(&status, const_cast<char *>(modsn.c_str()), const_cast<char *>(filename.c_str()));
  myThrow(&status);
}

std::string ConfigIPC::getModuleConfigurationString(const std::string modsn, const ABCDModule config) 
{
  sctConfStatus status;	
  status.returnCode = 0;

  std::string result = handle->getModuleConfigurationString(&status, const_cast<char *>(modsn.c_str()), copyABCDModuleToILU(config));
  myThrow(&status);

  return result;
}

void ConfigIPC::writePowerSupplyConfiguration(std::string filename)
{
  sctConfStatus status;	
  status.returnCode = 0;

  handle->writePowerSupplyConfiguration(&status, const_cast<char *>(filename.c_str()));
  myThrow(&status);
}

typedef sctConf_T_ilu__prefix_idlExceptionType_ConfigurationException ConfException;

void myThrow(sctConfStatus *status) 
{
  if(status->returnCode != 0) {
    if(status->returnCode == ilu::ProtocolError) {
      throw ConfigurationException("ILU Protocol Error!"); // + status->values.anyvalue
    }
    ConfException *e = status->values.sctConf_E_ConfigurationException_Value;
 
    throw ConfigurationException(e->detail);
  }
}
