#include <ipc/object.h>

#include "sctConf/configuration.h"
#include "sctConf.hh"

namespace SctConfiguration {

class ConfigIPC : public Configuration {
  ConfigIPC(const ConfigIPC &);
  ConfigIPC &operator=(const ConfigIPC &);

/*   static Configuration *m_config; */

/*   xmlDocPtr document; */
/*   xmlXPathContextPtr xpathContext; */

/*   Log log; */

  IPCObjectVar<sctConf_T_Configuration> handle;
public:
  ConfigIPC();
  ~ConfigIPC();

  std::list<unsigned int> listPartitions();
  std::list<unsigned int> listCratesInPartition(unsigned int partition);
  std::list<unsigned int> listRodsInCrate(unsigned int partition, unsigned int crate);
  std::list<unsigned int> listMURSInRod(unsigned int partition, unsigned int crate, unsigned int rod);
  std::list<std::string> listModulesInMUR(unsigned int partition, unsigned int MUR);
  std::list<std::string> listRModulesInMUR(unsigned int partition, unsigned int MUR);
  std::list<std::string> listAllModules();
  std::list<std::string> listUnusedModules();

  bool isDummyCrate(unsigned int partition, unsigned int crate);

  ABCDModule getModuleConfig(const std::string module);
  RodConfig getRodConfig(unsigned int partition, unsigned int crate, unsigned int rod);
  BOCChannelConfig getBOCConfig(unsigned int partition, unsigned int crate, unsigned int rod, 
                                unsigned int channel);
  BOCGlobalConfig getBOCGlobalConfig(unsigned int partition, unsigned int crate, unsigned int rod);
  TimConfig getTimConfig(unsigned int partition, unsigned int crate);
  char *getFibreMappings(unsigned int partition, unsigned int crate, unsigned int rod);
  void setFibreMapping(unsigned int partition, unsigned int crate, unsigned int rod, 
                       unsigned int channel, unsigned int tx, unsigned int rx0, unsigned int rx1);
  void printModuleConfig(const ABCDModule &conf);
  MURType getMURType(unsigned int MUR);
  void translateToROD(unsigned int MUR, unsigned int module,
		      unsigned int &partition, unsigned int &crate, 
		      unsigned int &rod, unsigned int &channel);
  void translateToRROD(unsigned int MUR, unsigned int module,
		       unsigned int &partition, unsigned int &crate, 
		       unsigned int &rod, unsigned int &channel);
  void translateToSN(unsigned int MUR, unsigned int module,
                     std::string &sn);
  void translateToBarrel(unsigned int MUR, unsigned int module,
                         unsigned int &barrel, unsigned int &row, int &number);
  void translateToEndcap(unsigned int MUR, unsigned int module,
                         unsigned int &disk, unsigned int &ring, unsigned int &number);
  void translateToPowerSupply(unsigned int MUR, unsigned int module,
                              unsigned int &partition, 
                              unsigned int &crate, unsigned int &channel);
  void translateFromROD(unsigned int partition, unsigned int crate, 
                        unsigned int rod, unsigned int channel,
                        unsigned int &MUR, unsigned int &module);
  void translateFromRROD(unsigned int partition, unsigned int crate,
                         unsigned int rod, unsigned int channel,
                         unsigned int &MUR, unsigned int &module);
  void translateFromSN(const std::string sn,
                       unsigned int &MUR, unsigned int &module);
  void translateFromBarrel(unsigned int barrel, unsigned int row, int number,
                           unsigned int &MUR, unsigned int &module);
  void translateFromEndcap(unsigned int disk, unsigned int ring, unsigned int number,
                           unsigned int &MUR, unsigned int &module);
  void translateFromPowerSupply(unsigned int partition, 
                                unsigned int crate, unsigned int channel, 
                                unsigned int &MUR, unsigned int &module);
  void configureModuleFromStructure(const std::string module, const ABCDModule conf);
  void configureModuleFromFile(const std::string filename);
  void configureROD(unsigned int partition, unsigned int crate, unsigned int rod, RodConfig conf);
  void configureTIM(unsigned int partition, unsigned int crate, TimConfig conf);
  void configureBOC(unsigned int partition, unsigned int crate, unsigned int rod, const BOCGlobalConfig &conf);
  void clearAll();
  void namePartition(int partition, const std::string name);
  void configureBOCChannel(int MUR, int position, const BOCChannelConfig &conf);
  void mapModuleMUR(int MUR, int order, int RMUR, int rorder, std::string number);
  void mapRODMUR(int partition, int crate, int rod, int order, int number);
  void mapBarrelMUR(int MUR, int barrel, int row, int position);
  void swapMURNames(int MUR1, int MUR2);
  void mapEndcapMUR(int MUR, int disk, int quadrant, int position);
  void mapPowerChannel(int MUR, int number, unsigned int partition, unsigned int crate, unsigned int channel);
  void modifyPowerParam(int MUR, int number, std::string name, float value);
  void modifyDefaultPowerParam(std::string name, float value);
  float getPowerParam(int MUR, int number, std::string name);
  void loadConfiguration(const std::string &filename);
  void saveConfiguration(const std::string filename = "");
  void saveModuleConfiguration(const std::string modsn, const std::string filename);
  std::string getModuleConfigurationString(const std::string modsn, const ABCDModule config);
  void writePowerSupplyConfiguration(std::string filename);
};

}
