package DisplayGUI;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import java.awt.Graphics2D.*;
import java.awt.geom.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;

class DisplayPanel extends JPanel implements guiInterface, TestDataInfo  {
        gui guiControl;
        private int selectedView = 0;
        FontMetrics metrics;
	int maxWidth;
        private int displayOption=0; // for testdata 0:rod 1: sctdb 2: rod-sctdb comparison

        private JMenu popupMenu;

        private Rectangle draggedRectangle;
        
        private boolean firstTime = true;
	private Rectangle area;   
        private Dimension dim;

        private ModuleCell redCell,lastClickedCell,popupCell;
        private String mouseLocationText;

        ConfigurationInterface config;
        
        boolean writeAccess;

    public DisplayPanel(gui parent){
        guiControl=parent;
        setBackground(Color.white);
        addMouseMotionListener(this);
        addMouseListener(this);
        dim = getSize();
        maxWidth = (int)dim.getWidth();

        setMinimumSize(new Dimension(14*boxWidth,yOffSet+58*boxHeight));
        setPreferredSize(new Dimension(14*boxWidth,yOffSet+58*boxHeight));    

        writeAccess=false;

        config = ConfigurationInterface.getInstance();
        config.setWindowSize(maxWidth,(int)dim.getHeight());

        getTestDataOptionMenu();
    }

    


    // Handles the event of the user pressing down the mouse button.
    public void mousePressed(MouseEvent e){
        if(!writeAccess) return;
        draggedRectangle = new Rectangle(new Point(e.getX(),e.getY()));   
    }

    // Handles the event of a user dragging the mouse while holding
    // down the mouse button.
    public void mouseDragged(MouseEvent e){
 
        if(!writeAccess || guiControl.displayStatus.getStatus()!=DisplayStatus.MODULE_GROUP_SELECTION) return;
        redCell=null;
        int width = e.getX() - (int)draggedRectangle.getX();
        int height = e.getY() - (int)draggedRectangle.getY();
        int thisModuleGroup = guiControl.selectorPane.getMouseClickIndex();
// re-define or resize the dragged rectangle
        if(width<0 || height<0) {
             int oldWidth = (int)draggedRectangle.getWidth();
             int oldHeight = (int)draggedRectangle.getHeight();
             draggedRectangle = new Rectangle(new Point(e.getX(),e.getY()),new Dimension(Math.abs(width)+oldWidth,Math.abs(height)+oldHeight));
             }
        else draggedRectangle.setSize(new Dimension(width,height));
        for(Enumeration e2 = config.getSerialNumberMap(selectedView).elements(); e2.hasMoreElements();) {
               ModuleCell thisCell = (ModuleCell) e2.nextElement();
               if(thisCell.getSctComponent()!=selectedView) continue;
               if(thisCell.intersects(draggedRectangle)) {
                   switch(e.getModifiers()) {
                     case InputEvent.BUTTON2_MASK:
                     case InputEvent.BUTTON3_MASK:
                     if(thisCell.getModuleGroup()==thisModuleGroup) thisCell.setModuleGroup(-1);
                     break;
                   default:
                       if(!thisCell.isSelected()) thisCell.setModuleGroup(thisModuleGroup);
                     }
               }
          }
        if(guiControl.displayStatus.getStatus()==DisplayStatus.MODULE_GROUP_SELECTION) setDisplayColors();
        repaint();

    }

    // Handles the event of a user releasing the mouse button.
    public void mouseReleased(MouseEvent e){
       if(draggedRectangle!=null && writeAccess) {
           draggedRectangle=null;
           repaint();
           if(guiControl.displayStatus.getStatus()==DisplayStatus.MODULE_GROUP_SELECTION) guiControl.tablesDisplayPane.updateTables();
       }
    }
       
     // This method is required by MouseListener.
     public void mouseMoved(MouseEvent e){ 
       if(draggedRectangle!=null) return;
       ModuleCell thisCell = config.getCell(selectedView,e.getX(),e.getY());
       if(thisCell==null) {
           redCell = null;
           if(mouseLocationText!=null) {
               mouseLocationText = null;
               repaint();
           }
           return;
       }

// check for redCell
       if(thisCell==redCell)  return;  // no change
       else {
//            if(redCell!=null) redCell.setSelected(false);
//            thisCell.setSelected(true);
            redCell = thisCell;
       }
// update mouse Text
       switch(guiControl.displayStatus.getStatus()) {
          case DisplayStatus.IS_MONITOR:
               if(guiControl.isInterface.testData!=null) {
                 mouseLocationText = new String(thisCell.getSerialNo()+" : "+thisCell.getData()+" data objects");
                 }
               break;
          case DisplayStatus.MODULE_GROUP_SELECTION:
               if(selectedView<4) mouseLocationText = new String(thisCell.getSerialNo()+" MUR: "+thisCell.getMUR()+" Position: "+thisCell.getModulePosition());
               else mouseLocationText = new String(thisCell.getSerialNo()+" MUR: "+thisCell.getMUR()+" Quadrant/Position: "+thisCell.getQuadrant()+"/"+thisCell.getEndCapPosition());
               break;
          case DisplayStatus.TEST_DATA:
          default:  
               mouseLocationText = new String(thisCell.getSerialNo()+" : "+thisCell.getData());
               break;
          }


       repaint();
     }

     // These methods are required by MouseMotionListener.
     public void mouseClicked(MouseEvent e){
       ModuleCell thisCell = config.getCell(selectedView,e.getX(),e.getY());
       if(thisCell==null) {
          switch(e.getModifiers()) {
           case InputEvent.BUTTON2_MASK:
           case InputEvent.BUTTON3_MASK:             
           guiControl.menuBar.showPopupMenu(e);
           break;
          default:
          }
         return;
        }
       
       lastClickedCell = thisCell;

       switch(guiControl.displayStatus.getStatus()) {

         case DisplayStatus.MODULE_GROUP_SELECTION:
            if(!writeAccess) return;
            int requestedModuleGroup =guiControl.selectorPane.getMouseClickIndex();  
              switch(e.getModifiers()) {
               case InputEvent.BUTTON2_MASK:
               case InputEvent.BUTTON3_MASK:     
               if(thisCell.getModuleGroup()==requestedModuleGroup) thisCell.setModuleGroup(-1);
               break;
              default:
               if(!thisCell.isSelected()) thisCell.setModuleGroup(requestedModuleGroup);
             }
           guiControl.tablesDisplayPane.updateTables();
           setDisplayColors();
           repaint();
           break;

       case DisplayStatus.IS_MONITOR:
           guiControl.tablesDisplayPane.listISObjects(thisCell);
           break;
       case DisplayStatus.TEST_DATA:
//              SummaryReader.ReaderReportUtilities.showTestReport(guiControl,thisCell);
           switch(e.getModifiers()) {
               case InputEvent.BUTTON2_MASK:
               case InputEvent.BUTTON3_MASK: 
                  popupCell=thisCell;    
                   JPopupMenu pMenu = popupMenu.getPopupMenu();
                   pMenu.show(e.getComponent(),e.getX(),e.getY());
                   pMenu.setInvoker(popupMenu);
               break;
              default:
           }
           break;
        default:
        }
       
       }

     public void mouseExited(MouseEvent e){}
     public void mouseEntered(MouseEvent e){}
                         

     public void refreshDisplay(){
          redCell=null;
	  repaint();
     }

    public void paintComponent(Graphics g){

        super.paintComponent(g);
        

        
        Graphics2D g2 = (Graphics2D)g;
	g2.setStroke(new BasicStroke(1.0f));

        if ( firstTime ) {
            dim = getSize();
            area = new Rectangle(dim);
            maxWidth = (int)dim.getWidth();
            firstTime = false;
            g2.setFont(displayTextFont);
            metrics = g2.getFontMetrics();
            config.setWindowSize(maxWidth,(int)dim.getHeight());
	}
        // Clears the rectangle that was previously drawn.
        g2.setPaint(Color.white);
        g2.fill(area);

        
        config.drawSCTViewOutline(g2,selectedView);
        drawTitle(g2);
        if(mouseLocationText!=null) drawMouseLocation(g2,mouseLocationText);
        drawGraphicsCells(g2);
        drawMouseSelections(g2);

    }


    
    public void drawGraphicsCells(Graphics2D g2) {
        ColorScale thisColorScale = guiControl.colorScalePane.getColorScale();
        if(selectedView<4) {
          for(Enumeration e = config.getSerialNumberMap(selectedView).elements(); e.hasMoreElements();) {
               ModuleCell thisCell = (ModuleCell) e.nextElement();
               if(thisCell.getSctComponent()!=selectedView) continue;
               g2.setPaint(thisCell.getColor());
               g2.fill((Polygon)thisCell);
               g2.setColor(Color.black);
               g2.draw((Polygon)thisCell);
               }
          }
         else {
          // for endcaps, must draw/paint middle ring first
          for(Enumeration e = config.getSerialNumberMap(selectedView).elements(); e.hasMoreElements();) {
               ModuleCell thisCell = (ModuleCell) e.nextElement();
               if(thisCell.getSctComponent()!=selectedView) continue;
               int ring = thisCell.getPosition();
               if(ring<13 || ring>22) continue;
               g2.setPaint(thisCell.getColor());
               g2.fill((Polygon)thisCell);
               g2.setColor(Color.black);
               g2.draw((Polygon)thisCell);
               }
          for(Enumeration e = config.getSerialNumberMap(selectedView).elements(); e.hasMoreElements();) {
               ModuleCell thisCell = (ModuleCell) e.nextElement();
               if(thisCell.getSctComponent()!=selectedView) continue;
               int ring = thisCell.getPosition();
               if(ring>=13 && ring<=22) continue;
               g2.setPaint(thisCell.getColor());
               g2.fill((Polygon)thisCell);
               g2.setColor(Color.black);
               g2.draw((Polygon)thisCell);
               }
           }
    }

   public void drawMouseSelections(Graphics2D g2) {
       if(redCell!=null) {
            g2.setColor(Color.red);
            g2.draw((Polygon)redCell);
          }
           if(draggedRectangle!=null) {
               g2.setColor(Color.red);
               g2.draw(draggedRectangle);
           }
          g2.setColor(Color.lightGray);
      }



    public void drawMouseLocation(Graphics2D g2, String mouseLocationText) {
        g2.setFont(displayTextFont);
//        g2.setPaint(darkBlueColor);
        g2.setPaint(Color.white);
        metrics = g2.getFontMetrics();
        int stringWidth = metrics.stringWidth(mouseLocationText);
        int stringHeight = metrics.getHeight();
        g2.drawString(mouseLocationText,maxWidth/2+7*boxWidth - stringWidth-5,yOffSet+boxHeight-stringHeight);
    }

    public void drawTitle(Graphics2D g2) {
        g2.setPaint(Color.blue);
        int xText = maxWidth/2-6*boxWidth;
        Rectangle top = new Rectangle(xText,0,13*boxWidth,yOffSet);
        g2.fill(top);
        xText+=5;
        String statusString = DisplayStatus.statusNames[guiControl.displayStatus.getStatus()];

        g2.setFont(displayTextFont);
        g2.setPaint(Color.white);
        String viewString = "View: "+SCTViewNames[selectedView];
        g2.drawString(viewString,xText,yOffSet+boxHeight-g2.getFontMetrics().getHeight());
        
        String runScanString;
        String onClickString;

        switch(guiControl.displayStatus.getStatus()) {
             case DisplayStatus.MODULE_GROUP_SELECTION:
                g2.setFont(displayTitleFont);
                g2.setPaint(Color.yellow);
                metrics = g2.getFontMetrics();
                g2.drawString(statusString,xText,metrics.getHeight());
                g2.setFont(displayTextFont);
                g2.setPaint(Color.yellow);
                
                String[] cols = {"Group2","Group1","Modules"};
                String[] values = {Integer.toString(guiControl.tablesDisplayPane.getRowCount(1)),Integer.toString(guiControl.tablesDisplayPane.getRowCount(0)),Integer.toString(config.getNumberOfModules(selectedView))};
                int width=80;
                int xMax=maxWidth/2+7*boxWidth;
                for(int i=0;i<cols.length;i++) {
                    int xloc = xMax - i*width - g2.getFontMetrics().stringWidth(cols[i])-5;
                    int yloc = g2.getFontMetrics().getHeight();
                    g2.drawString(cols[i],xloc,yloc);
                    xloc =  xMax - i*width - g2.getFontMetrics().stringWidth(values[i])-5;
                    yloc+=yloc;
                    g2.drawString(values[i],xloc,yloc-2);
                    }
                 break;


             case DisplayStatus.TEST_DATA:
                int displayedTestIndex = guiControl.testDataInterface.getListFilterIndex();
                int selectedTestIndex = guiControl.testDataInterface.getTestIndex();
                if(selectedTestIndex>=0) statusString=testMenuNames[selectedTestIndex]+" Result";
                else if(displayedTestIndex>=0) statusString=testMenuNames[displayedTestIndex]+" Result";
                g2.setFont(displayTitleFont);
                g2.setPaint(Color.yellow);
                metrics = g2.getFontMetrics();
                g2.drawString(statusString,xText,metrics.getHeight());
                int runno = guiControl.testDataInterface.getRunNo();
                runScanString = "No data";
                int runStringWidth;
                g2.setFont(displayTitleFont);
                metrics = g2.getFontMetrics();
                switch(displayOption) {
                case 0:
                  if(runno!=-1) runScanString = "Run "+Integer.toString(runno)+" Scan "+Integer.toString(guiControl.testDataInterface.getScanNo());
                  g2.setPaint(Color.yellow);
                  runStringWidth = g2.getFontMetrics().stringWidth(runScanString);
                  g2.drawString(runScanString,maxWidth/2+7*boxWidth - runStringWidth-5,g2.getFontMetrics().getHeight()); 
                  String dataDescription;
                  if((dataDescription = guiControl.selectorPane.getDisplayParameter())!=null) {
                    g2.setPaint(Color.white);
                    int stringWidth = g2.getFontMetrics().stringWidth(dataDescription);
                    g2.drawString(dataDescription,maxWidth/2+7*boxWidth - runStringWidth-20-stringWidth,g2.getFontMetrics().getHeight()); 
                    }
                  break;
                case 1:
                  runScanString = "SCTDB Reference Data";
                  g2.setPaint(Color.red);
                  runStringWidth = g2.getFontMetrics().stringWidth(runScanString);
                  g2.drawString(runScanString,maxWidth/2+7*boxWidth - runStringWidth-5,g2.getFontMetrics().getHeight()); 
                  break;
                case 2:
                  runScanString = "Comparison with SCTDB";
                  g2.setPaint(Color.red);
                  runStringWidth = g2.getFontMetrics().stringWidth(runScanString);
                  g2.drawString(runScanString,maxWidth/2+7*boxWidth - runStringWidth-5,g2.getFontMetrics().getHeight()); 
                  break;
                default:
                  runScanString = "No Data";
                  g2.setPaint(Color.white);
                  runStringWidth = g2.getFontMetrics().stringWidth(runScanString);
                  g2.drawString(runScanString,maxWidth/2+7*boxWidth - runStringWidth-5,g2.getFontMetrics().getHeight()); 
                }
                g2.setFont(displayTextFont);
                g2.setPaint(Color.white);
                String optionString;
                if((optionString=guiControl.selectorPane.getOption())!=null) {
                  int stringWidth = g2.getFontMetrics().stringWidth(optionString);
                  g2.drawString(optionString,maxWidth/2-stringWidth/2-20,yOffSet+boxHeight-g2.getFontMetrics().getHeight());
                  }
                break;
             case DisplayStatus.SCTDB_DATA:
                displayedTestIndex = guiControl.sctdbDataInterface.getListFilterIndex();
                selectedTestIndex = guiControl.sctdbDataInterface.getTestIndex();
                if(selectedTestIndex>=0) statusString=testMenuNames[selectedTestIndex]+" (SCTDB)";
                else if(displayedTestIndex>=0) statusString=testMenuNames[displayedTestIndex]+" (SCTDB)";
                g2.setFont(displayTitleFont);
                g2.setPaint(Color.yellow);
                metrics = g2.getFontMetrics();
                g2.drawString(statusString,xText,metrics.getHeight());
                runno = guiControl.sctdbDataInterface.getRunNo();
                runScanString = "Reference";
                g2.setFont(displayTitleFont);
                g2.setPaint(Color.red);
                metrics = g2.getFontMetrics();
                runStringWidth = g2.getFontMetrics().stringWidth(runScanString);
                g2.drawString(runScanString,maxWidth/2+7*boxWidth - runStringWidth-5,g2.getFontMetrics().getHeight());  
                String dataDescription;
                if((dataDescription = guiControl.selectorPane.getDisplayParameter())!=null) {
                  g2.setPaint(Color.white);
                  int stringWidth = g2.getFontMetrics().stringWidth(dataDescription);
                  g2.drawString(dataDescription,maxWidth/2+7*boxWidth - runStringWidth-20-stringWidth,g2.getFontMetrics().getHeight());
                  }
                if((optionString=guiControl.selectorPane.getOption())!=null) {
                  g2.setFont(displayTextFont);
                  int stringWidth = g2.getFontMetrics().stringWidth(optionString);
                  g2.drawString(optionString,maxWidth/2-stringWidth/2-20,yOffSet+boxHeight-g2.getFontMetrics().getHeight());
                  }
                break;
             case DisplayStatus.IS_MONITOR:
                g2.setFont(displayTitleFont);
                g2.setPaint(Color.yellow);
                metrics = g2.getFontMetrics();
                g2.drawString(statusString,xText,metrics.getHeight());
                runno = guiControl.isInterface.getRunNo();
                runScanString = "No data";
                if(runno!=-1) runScanString = "Run "+Integer.toString(runno)+" Scan "+Integer.toString(guiControl.isInterface.getScanNo());
                g2.setFont(displayTitleFont);
                g2.setPaint(Color.yellow);
                metrics = g2.getFontMetrics();
                int stringWidth = g2.getFontMetrics().stringWidth(runScanString);
                g2.drawString(runScanString,maxWidth/2+7*boxWidth - stringWidth-5,g2.getFontMetrics().getHeight());   
                if((dataDescription = guiControl.selectorPane.getDisplayParameter())!=null) {
                  g2.setPaint(Color.white);
                  stringWidth = g2.getFontMetrics().stringWidth(dataDescription);
                  g2.drawString(dataDescription,maxWidth/2-stringWidth/2,g2.getFontMetrics().getHeight());
                  }
                break;
             default:
                g2.setFont(displayTitleFont);
                g2.setPaint(Color.yellow);
                metrics = g2.getFontMetrics();
                g2.drawString(statusString,xText,metrics.getHeight());
           }
    }

    public void setWriteAccess(boolean isWriteAble) {
        writeAccess = isWriteAble;
    }
    public void setSCTView(int view) {
        selectedView=view;
        setDisplayColors();
        refreshDisplay();
        guiControl.tablesDisplayPane.updateTables();
        }
    public void setDisplayOption(int displayOption) {
        this.displayOption=displayOption;
        guiControl.colorScalePane.setColorScale();
        setDisplayColors();
        refreshDisplay();
        guiControl.tablesDisplayPane.listTestResults();
        }
   public int getDisplayOption() {
        return displayOption;
        }
   public void resetDisplayOption() {
        displayOption=0;
        }
   public int getSCTView() {
        return selectedView;
        }
   public ModuleCell getLastClickedCell() {
        return lastClickedCell;
        }

    public void setDisplayColors() {
//       System.out.println("**setDisplayColors**");
       String theData;
       switch(guiControl.displayStatus.getStatus()) {
             case DisplayStatus.TEST_DATA:
               String header = guiControl.testDataInterface.getResultObjectHeader();
               int testIndex = guiControl.testDataInterface.getTestIndex();
               MODULELOOP:
               for(Enumeration e = config.getSerialNumberMap(selectedView).elements(); e.hasMoreElements();) {
                  ModuleCell thisCell = (ModuleCell) e.nextElement();
                  if(header==null) {
                       thisCell.setData("No Data",java.awt.Color.white);
                       continue MODULELOOP;
                       }    
                  String sn = thisCell.getSerialNo();
                  switch(displayOption) {
                  case 1: // SCTDB Data only    
                    header = guiControl.testDataInterface.getSCTDBObjectHeader();
                    if((theData = getSCTDBData(header+sn,testIndex))==null) {
                       thisCell.setData("No Data",java.awt.Color.white);
                       continue MODULELOOP;
                       }    
                     break;
                   case 2: // Compare SCTDB and ROD data
                    if((theData=getTestData(header+sn,testIndex))==null) {
                       thisCell.setData("No Data",java.awt.Color.white);
                       continue MODULELOOP;
                       }  
                    String sctdbData;
                    
                    if((sctdbData = getSCTDBData(guiControl.testDataInterface.getSCTDBObjectHeader()+sn,testIndex))==null) {
                       thisCell.setData("No Data",java.awt.Color.white);
                       continue MODULELOOP;
                       }    
                    double rod_Data = Double.parseDouble(theData);
                    double sctdb_Data = Double.parseDouble(sctdbData);
                    double diff = Math.abs(rod_Data-sctdb_Data);
                    if(testIndex==TEST_FULLBYPASS && guiControl.selectorPane.getDisplayParameterIndex()>2) theData="1";
                    else theData = (Math.abs(rod_Data-sctdb_Data)>testDataTolerances[testIndex][guiControl.selectorPane.getDisplayParameterIndex()]) ? "2" : "1";
                    break;
                  case 0: // ROD Data only
                  default:
                    if((theData=getTestData(header+sn,testIndex))==null) {
                       thisCell.setData("No Data",java.awt.Color.white);
                       continue MODULELOOP;
                       }    
                  }

                 // update the colors           
                 ColorScale thisColorScale;
                 if((thisColorScale = guiControl.colorScalePane.getColorScale())!=null) {
                       thisCell.setData(theData,thisColorScale.getColor(Double.parseDouble(theData)));
                  }
                  else thisCell.setData("No Data",java.awt.Color.white);
                  }
                break;
             case DisplayStatus.SCTDB_DATA:
               header = guiControl.sctdbDataInterface.getResultObjectHeader();
               testIndex = guiControl.sctdbDataInterface.getTestIndex();
               // loop through all modules, reading the summary string for each
               for(Enumeration e = config.getSerialNumberMap(selectedView).elements(); e.hasMoreElements();) {
                  ModuleCell thisCell = (ModuleCell) e.nextElement();
                  if(header==null) {
                       thisCell.setData("No Data",java.awt.Color.white);
                       continue;
                       }    
                  if((theData = getSCTDBData(header+thisCell.getSerialNo(),testIndex))==null) {
                     thisCell.setData("No Data",java.awt.Color.white);
                     continue;
                     }
                 // update the colors           
                 ColorScale thisColorScale;
                 if((thisColorScale = guiControl.colorScalePane.getColorScale())!=null) {
                       if(theData!=null) thisCell.setData(theData,thisColorScale.getColor(Double.parseDouble(theData)));
                       else thisCell.setData("No Data",java.awt.Color.white);
                  }
                  else thisCell.setData("No Data",java.awt.Color.white);
                  }
                break;
             case DisplayStatus.IS_MONITOR:
                for(Enumeration e = config.getSerialNumberMap(selectedView).elements(); e.hasMoreElements();) {
                        ModuleCell thisCell = (ModuleCell) e.nextElement();
                        if(guiControl.isInterface.testData==null) {
                            thisCell.setData("No data",java.awt.Color.white);
                            continue;
                            }
                        double value = (double)guiControl.isInterface.getNumberOfISObjects(thisCell.getSerialNo()); 
                        thisCell.setData(Integer.toString(guiControl.isInterface.getNumberOfISObjects(thisCell.getSerialNo())),guiControl.colorScalePane.getColorScale().getColor(value));
//                        g2.setPaint(thisColorScale.getColor(value)); 
                        }
                      break;

             case DisplayStatus.MODULE_GROUP_SELECTION:   
               for(Enumeration e = config.getSerialNumberMap(selectedView).elements(); e.hasMoreElements();) {
                  ModuleCell thisCell = (ModuleCell) e.nextElement();
                  int thisModuleGroup = thisCell.getModuleGroup();                   
                  thisCell.setData(Integer.toString(thisModuleGroup),guiControl.colorScalePane.getColorScale().getColor((double)thisModuleGroup));
                  }
                break;

             default:
           }
       }

     private String getTestData(String resultObject,int testIndex) {
                  if(testIndex==-1) return null;
                  String theData;
                  if(!Sct.IS.SctNames.getISRepository().contains(resultObject)) return null;
                  TestSummaryIS result = new TestSummaryIS(); 
                  Sct.IS.SctNames.getISRepository().getValue(resultObject, result); 
                  int parameterIndex = guiControl.selectorPane.getDisplayParameterIndex();
                  if(parameterIndex<0) return null;
                  SummaryReader.Reader reader;     
//
 
                  switch(parameterIndex) {
                    case 0: // test status
                      reader = new SummaryReader.Reader(result.dataString,false,true);
                      theData = reader.getTest().getTestStatus();  
                      break;
                    case 1: // defect count
                      reader = new SummaryReader.Reader(result.dataString,true,false);
                      theData = reader.getTest().getDefectiveStripCount();  
                      break;
                    default:
                      reader = new SummaryReader.Reader(result.dataString,testIndex,parameterIndex-2);
                      theData = reader.getData(guiControl.selectorPane.getMouseClickIndex());
                    }
                   return theData;
                   }

     private String getSCTDBData(String resultObject, int testIndex) {
             if(testIndex==-1) return null;
             String theData;
                  if(!Sct.IS.SctNames.getISRepository().contains(resultObject)) return null;
                  SCTDB_Data result = new SCTDB_Data(); 
                  Sct.IS.SctNames.getISRepository().getValue(resultObject, result); 
                  int parameterIndex = guiControl.selectorPane.getDisplayParameterIndex();
                  if(parameterIndex<0) return null;
                  switch(parameterIndex) {
                  case 0:
                      theData=Integer.toString(result.status);
                      break;
                  case 1:
                      theData = Integer.toString(result.noDefectChannels); 
                      break;                  
                 default:
                    theData = extractSCTDBData(result, guiControl.selectorPane.getMouseClickIndex(), parameterIndex-2);
                    }
              return theData;
              }
    

      private String extractSCTDBData(SCTDB_Data result, int index, int parameterIndex) {
              String theData;
              switch(index) {
                      case -1:
                         double worstValue=0.;
                         double thisValue = Double.parseDouble(result.data_m0[parameterIndex]);
                         if(thisValue>worstValue) worstValue=thisValue;
                         thisValue = Double.parseDouble(result.data_s1[parameterIndex]);
                         if(thisValue>worstValue) worstValue=thisValue;
                         thisValue = Double.parseDouble(result.data_s2[parameterIndex]);
                         if(thisValue>worstValue) worstValue=thisValue;
                         thisValue = Double.parseDouble(result.data_s3[parameterIndex]);
                         if(thisValue>worstValue) worstValue=thisValue;
                         thisValue = Double.parseDouble(result.data_s4[parameterIndex]);
                         if(thisValue>worstValue) worstValue=thisValue;
                         thisValue = Double.parseDouble(result.data_e5[parameterIndex]);
                         if(thisValue>worstValue) worstValue=thisValue;
                         thisValue = Double.parseDouble(result.data_m8[parameterIndex]);
                         if(thisValue>worstValue) worstValue=thisValue;
                         thisValue = Double.parseDouble(result.data_s9[parameterIndex]);
                         if(thisValue>worstValue) worstValue=thisValue;
                         thisValue = Double.parseDouble(result.data_s10[parameterIndex]);
                         if(thisValue>worstValue) worstValue=thisValue;
                         thisValue = Double.parseDouble(result.data_s11[parameterIndex]);
                         if(thisValue>worstValue) worstValue=thisValue;
                         thisValue = Double.parseDouble(result.data_s12[parameterIndex]);
                         if(thisValue>worstValue) worstValue=thisValue;
                         thisValue = Double.parseDouble(result.data_e13[parameterIndex]);
                         if(thisValue>worstValue) worstValue=thisValue;
                         theData = Double.toString(thisValue);
                         break;
                      case 0:
                         theData = result.data_m0[parameterIndex];
                         break;
                      case 1:
                         theData = result.data_s1[parameterIndex];
                         break;            
                      case 2:
                         theData = result.data_s2[parameterIndex];
                         break;          
                      case 3:
                         theData = result.data_s3[parameterIndex];
                         break;
                      case 4:
                         theData = result.data_s4[parameterIndex];
                         break;
                      case 5:
                         theData = result.data_e5[parameterIndex];
                         break;
                      case 6:
                         theData = result.data_m8[parameterIndex];
                         break;
                      case 7:
                         theData = result.data_s9[parameterIndex];
                         break;
                      case 8:
                         theData = result.data_s10[parameterIndex];
                         break;
                      case 9:
                         theData = result.data_s11[parameterIndex];
                         break;
                      case 10:
                         theData = result.data_s12[parameterIndex];
                         break;
                      case 11:
                         theData = result.data_e13[parameterIndex];
                         break;
                       default:
                         theData=null;
                       } 
             return theData;
             }

   private void getTestDataOptionMenu() {
       popupMenu = new JMenu("Test Data Options");
       popupMenu.setFont(menuFont);
       JMenuItem item1 = new JMenuItem("Test Report");
       item1.setFont(menuFont);
       popupMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
//                System.out.println("Selected "+popupCell.getSerialNo());
              SummaryReader.ReaderReportUtilities.showTestReport(guiControl,popupCell);
              }
          });  
       item1 = new JMenuItem("Summary Plots");
       item1.setFont(menuFont);
       popupMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                  String objectname;
                  if((objectname = guiControl.testDataInterface.getTestObjectHeader())!=null) {
                     guiControl.isInterface.launchExternalViewer(objectname+popupCell.getSerialNo());
                     }
                 }
          });  
       item1 = new JMenuItem("Module Configuration");
       item1.setFont(menuFont);
       popupMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
            JFrame frame2 = new JFrame("Module Configuration");                    
            frame2.getContentPane().add(new TestGUI.ModuleConfigurationPanel(popupCell.getSerialNo()));
            frame2.pack();
            frame2.show();
                     }

          });  
       item1 = new JMenuItem("Display All SCTDB Data");
       item1.setFont(menuFont);
       popupMenu.add(item1);
       item1.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
             String sn = popupCell.getSerialNo();
             if(sn.equals("SCTTestAPI_PseudoModule")) sn = "20220330200020";
             ProdDatabase.ReportDownloader d = new ProdDatabase.ReportDownloader(guiUtilities.HTMLViewer.MODULESUMMARY,sn,0);
             if(d.isOkToStart()) d.start();
             }

          });  
       }
 
}
