package DisplayGUI;

import java.io.*;
import java.util.*;
import is.*;
import Sct.Serializable;
import Sct.IS.*;
//import SctControl.*;
import java.sql.*;
import ProdDatabase.SCTDBInterface;
import ProdDatabase.SCTDBInfo;
/*
 * SCTDBTestDownloader.java
 *
 */

/**
 *
 * @author  robinson
 */
public class SCTDBTestDownloader extends Thread implements SCTDBInfo, DisplayGUI.TestDataInfo {
    Vector listOfModules;
    int testIndex;
    String location;
    boolean isWarm;
    boolean refresh;
    boolean download;

    public SCTDBTestDownloader(Vector listOfModules, int testIndex, String location, boolean isWarm, boolean refresh) {
         super("SCTDB_DOWNLOAD");
         this.listOfModules=listOfModules;
         this.testIndex=testIndex;
         this.location=location;
         this.isWarm=isWarm;
         this.refresh=refresh;

         download=true;

         for(int i=listOfModules.size()-1;i>=0;i--) {
            String module = (String)listOfModules.elementAt(i);
            String objectName = "SCTDBData."+testNames[testIndex]+"."+location+"."+(String)listOfModules.elementAt(i);
            // if ANY sctdb test object exists, and not requested a 'refresh' then don't download anything
            if(SctNames.getISRepository().contains(objectName) && !refresh) {
                download=false;
                break;
                }
            }

         if(testIndex==TEST_NMASK) download=false; // no such sctdaq test

         setPriority(Thread.MIN_PRIORITY);

         }
    public boolean downloadsRequired() {
         return download;
         }
    public boolean isOkToStart() {
         Thread[] allThreads = new Thread[Thread.activeCount()];
         Thread.enumerate(allThreads);
         for(int i=0;i<allThreads.length;i++) {
            if(allThreads[i]!=null && allThreads[i].getName().equals("SCTDB_DOWNLOAD") && allThreads[i].isAlive()) {
                  System.out.println("***A download from SCTDB is already in progress. Please wait...");
                  return false;
                  }
            }
         return true;
         }

    public void run() {
            
         Hashtable snHash = new Hashtable();
         Hashtable testnoHash = new Hashtable();

         StringBuffer moduleList = new StringBuffer();
         for(int i=0;i<listOfModules.size();i++) {
              if(i>0) moduleList.append(",");
              String module = (String)listOfModules.elementAt(i);
              if(module.equals("SCTTestAPI_PseudoModule")) moduleList.append("20220330200020");
              else moduleList.append(module);
              }
         System.out.println("SCTDB: Downloading "+listOfModules.size()+" module "+testNames[testIndex]+" tests.");
         StringBuffer sqlStat = new StringBuffer();
            sqlStat = new StringBuffer("SELECT tests.ser_no,tests.pass,tests.problem,tests.TEST_no,tests.locn_name,tests.TEST_date");
            switch(testIndex) {
                    case SCTDB_TEST_RESET:
                    case SCTDB_TEST_REDUNDANCY:
                    case SCTDB_TEST_LONGTERM:
                    case SCTDB_TEST_IV:
                        break;
                    default:
                        for(int y=0;y<chipDBnames.length;y++) {
                           for(int paramIndex=0;paramIndex<DBChipParameterNames[testIndex].length;paramIndex++) {
                                  sqlStat.append(","+chipDBnames[y]+DBChipParameterNames[testIndex][paramIndex]);
                                  }
                          }

                     }
         sqlStat.append(" FROM "+sctdaqDBTableNames[testIndex]+",tests,sct_tstdcsinfo");
         sqlStat.append(" WHERE tests.TEST_name = '"+sctdaqDBTestNames[testIndex]+"'");
         sqlStat.append(" AND tests.ser_no IN ("+moduleList.toString()+")");
         if(!location.equals("Any")) sqlStat.append(" AND tests.locn_name LIKE '"+location+"'");
         sqlStat.append(" AND tests.TEST_no = "+sctdaqDBTableNames[testIndex]+".TEST_no");
         if(isWarm) sqlStat.append(" AND sct_tstdcsinfo.t0>10 AND sct_tstdcsinfo.TEST_no=tests.TEST_no"); 
         else  sqlStat.append(" AND sct_tstdcsinfo.t0<=10 AND sct_tstdcsinfo.TEST_no=tests.TEST_no"); 
         sqlStat.append(" ORDER BY tests.TEST_date DESC,tests.TEST_no DESC");

         try {

         Statement statement = SCTDBInterface.getInstance().connection.createStatement();
//         System.out.println(sqlStat.toString());
         ResultSet resultSet = statement.executeQuery(sqlStat.toString());
         for(boolean n = resultSet.next() ; n==true ; n=resultSet.next() ){
            int rc=1;

            String sn = resultSet.getString(rc++);
//            if(sn.equals("20220330200020")) sn="SCTTestAPI_PseudoModule";
            if(snHash.containsKey(sn)) continue;
            int status = (resultSet.getString(rc++).equals("YES")) ? 0 : 2;
            if(status==0 && resultSet.getString(rc++).equals("YES")) status = 1;

            SCTDB_Data sctdbData = new SCTDB_Data(); 
            sctdbData.status=status; // 0: pass, 1: problem, 2: fail
            sctdbData.testIndex=testIndex;
            sctdbData.testNumber = resultSet.getString(rc++);
            sctdbData.testLocation = resultSet.getString(rc++);
            sctdbData.testDate = guiUtilities.DaveUtils.extractDate(resultSet.getString(rc++));
            System.out.println("Downloading "+testNames[testIndex]+" for "+sn+" taken by "+sctdbData.testLocation+" on "+sctdbData.testDate);

            sctdbData.data_m0 = new String[DBChipParameterNames[testIndex].length];
            for(int r=0;r<DBChipParameterNames[testIndex].length;r++) sctdbData.data_m0[r]=resultSet.getString(rc++);
            sctdbData.data_s1 = new String[DBChipParameterNames[testIndex].length];
            for(int r=0;r<DBChipParameterNames[testIndex].length;r++) sctdbData.data_s1[r]=resultSet.getString(rc++);
            sctdbData.data_s2 = new String[DBChipParameterNames[testIndex].length];
            for(int r=0;r<DBChipParameterNames[testIndex].length;r++) sctdbData.data_s2[r]=resultSet.getString(rc++);
            sctdbData.data_s3 = new String[DBChipParameterNames[testIndex].length];
            for(int r=0;r<DBChipParameterNames[testIndex].length;r++) sctdbData.data_s3[r]=resultSet.getString(rc++);
            sctdbData.data_s4 = new String[DBChipParameterNames[testIndex].length];
            for(int r=0;r<DBChipParameterNames[testIndex].length;r++) sctdbData.data_s4[r]=resultSet.getString(rc++);
            sctdbData.data_e5 = new String[DBChipParameterNames[testIndex].length];
            for(int r=0;r<DBChipParameterNames[testIndex].length;r++) sctdbData.data_e5[r]=resultSet.getString(rc++);
            sctdbData.data_m8 = new String[DBChipParameterNames[testIndex].length];
            for(int r=0;r<DBChipParameterNames[testIndex].length;r++) sctdbData.data_m8[r]=resultSet.getString(rc++);
            sctdbData.data_s9 = new String[DBChipParameterNames[testIndex].length];
            for(int r=0;r<DBChipParameterNames[testIndex].length;r++) sctdbData.data_s9[r]=resultSet.getString(rc++);
            sctdbData.data_s10 = new String[DBChipParameterNames[testIndex].length];
            for(int r=0;r<DBChipParameterNames[testIndex].length;r++) sctdbData.data_s10[r]=resultSet.getString(rc++);
            sctdbData.data_s11 = new String[DBChipParameterNames[testIndex].length];
            for(int r=0;r<DBChipParameterNames[testIndex].length;r++) sctdbData.data_s11[r]=resultSet.getString(rc++);
            sctdbData.data_s12 = new String[DBChipParameterNames[testIndex].length];
            for(int r=0;r<DBChipParameterNames[testIndex].length;r++) sctdbData.data_s12[r]=resultSet.getString(rc++);
            sctdbData.data_e13 = new String[DBChipParameterNames[testIndex].length];
            for(int r=0;r<DBChipParameterNames[testIndex].length;r++) sctdbData.data_e13[r]=resultSet.getString(rc++);

// painful conversion from number of good channels to number of bad channels
            int index=0;
            switch(testIndex) {
               case SCTDB_TEST_PIPELINE:
                 convertData(sctdbData,0);
                 break;
               case SCTDB_TEST_TRIM:
                 convertData(sctdbData,2);
                 break;
               default:
               }


            snHash.put(sn,sctdbData);
            testnoHash.put(sctdbData.testNumber,new Hashtable()); // to be filled with the no of defects in next query
            }
        statement.close();

        if(snHash.size()==0) {
            System.out.println("No Data available from SCTDB");
            return;
            }
        StringBuffer temp = new StringBuffer();
        int count=0;
        for(Enumeration e = testnoHash.keys(); e.hasMoreElements();) {
            if(count>0) temp.append(",");
            count++;
            temp.append((String)e.nextElement());
            }
        sqlStat = new StringBuffer();
        sqlStat.append("SELECT defects.TEST_no,defects.chan_1st,defects.chan_last");
        sqlStat.append(" FROM defects");
        sqlStat.append(" WHERE defects.TEST_no IN ("+temp.toString()+")");


        statement = SCTDBInterface.getInstance().connection.createStatement();
//         System.out.println(sqlStat.toString());
        resultSet = statement.executeQuery(sqlStat.toString());

        for(boolean n = resultSet.next() ; n==true ; n=resultSet.next() ){ 
            String thisTestNo = resultSet.getString(1);
            Hashtable tempHash = (Hashtable)testnoHash.get(thisTestNo);
            String chan1 = resultSet.getString(2);
            String chanLast = resultSet.getString(3);
            System.out.println("defects: "+thisTestNo+":"+chan1+"-"+chanLast);
            int i1 = Integer.parseInt(chan1);
            int i2 = Integer.parseInt(chanLast);
            for(int i=i1;i<=i2;i++) tempHash.put(thisTestNo,Integer.toString(i));
            }
        statement.close();
        for(Enumeration e = snHash.keys(); e.hasMoreElements();) {
            String sn = (String)e.nextElement();
            SCTDB_Data sctdbData = (SCTDB_Data)snHash.get(sn);
            sctdbData.noDefectChannels = ((Hashtable)testnoHash.get(sctdbData.testNumber)).size();   
            if(sn.equals("20220330200020")) sn="SCTTestAPI_PseudoModule";           
            String objectName = "SCTDBData."+testNames[testIndex]+"."+location+"."+sn;
            if(SctNames.getISRepository().contains(objectName)) {
                if(!refresh) continue;
                SctNames.getISRepository().remove(objectName);
                }
            SctNames.getISRepository().insert(objectName, sctdbData);  
            System.out.println("SCTDB Object published as "+objectName);        
            }


        int notDownloaded = listOfModules.size()-snHash.size();
        if(notDownloaded>0) {
           for(int i=0;i<listOfModules.size();i++) {
              String module = (String)listOfModules.elementAt(i);
              if(!snHash.containsKey(module) && !module.equals("SCTTestAPI_PseudoModule")) System.out.println("No "+testNames[testIndex]+" data available for "+(String)listOfModules.elementAt(i));
              }
           }

        }catch(Exception e) {System.out.println("Failed to publish SCTDB Data: "+e.toString());}


   }

   private void convertData(SCTDB_Data sctdbData, int index) {
               sctdbData.data_m0[index]=Integer.toString(128-Integer.parseInt(sctdbData.data_m0[index]));
               sctdbData.data_s1[index]=Integer.toString(128-Integer.parseInt(sctdbData.data_s1[index]));
               sctdbData.data_s2[index]=Integer.toString(128-Integer.parseInt(sctdbData.data_s2[index]));
               sctdbData.data_s3[index]=Integer.toString(128-Integer.parseInt(sctdbData.data_s3[index]));
               sctdbData.data_s4[index]=Integer.toString(128-Integer.parseInt(sctdbData.data_s4[index]));
               sctdbData.data_e5[index]=Integer.toString(128-Integer.parseInt(sctdbData.data_e5[index]));
               sctdbData.data_m8[index]=Integer.toString(128-Integer.parseInt(sctdbData.data_m8[index]));
               sctdbData.data_s9[index]=Integer.toString(128-Integer.parseInt(sctdbData.data_s9[index]));
               sctdbData.data_s10[index]=Integer.toString(128-Integer.parseInt(sctdbData.data_s10[index]));
               sctdbData.data_s11[index]=Integer.toString(128-Integer.parseInt(sctdbData.data_s11[index]));
               sctdbData.data_s12[index]=Integer.toString(128-Integer.parseInt(sctdbData.data_s12[index]));
               sctdbData.data_e13[index]=Integer.toString(128-Integer.parseInt(sctdbData.data_e13[index]));
               }

}
