package DisplayGUI;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import java.util.regex.*;
import is.*;
import Sct.Serializable;
import Sct.IS.*;
import Sct_CalibrationController.TestData;
import GuiComponents.System.*;

/*
 * TestDataInterface.java
 *
 */

/**
 *
 * @author  robinson
 */
public class TestDataInterface implements TestDataInfo, InfoListener {
    
    gui guiControl;
    Hashtable serverHash;  // hash to keep track of which servers we are subscribed to...
    String theObjectName;
    Vector ISTests;
    int testIndex;            // the currently selected test
    int listFilterIndex;      // the requested type of test
    TestData testData;

    Pattern objectPattern;

    String resultObjectPattern;  // regex for analysis chain objects
    String serversPattern;        //regex for servers used by analysis chain
    String controlPattern;
    String testsPattern;
    String resultObjectHeader;   // header of the current selected summary result object name
    String sctdbObjectHeader;    // header of reference sctdb data object
    String testObjectHeader;     // header of currently selected test object name

    public TestDataInterface(gui parent) {
        guiControl=parent;
//        sct = SCTInterface2.getInstance();
        is = GuiComponents.System.ISInterface.getInstance();
        serverHash = new Hashtable();
        ISTests = new Vector();
        controlPattern = ".*TestData\\.[0-9]+\\.[0-9]+";
        objectPattern = Pattern.compile(controlPattern);
        resultObjectPattern = ".*(Summary).*";
        testIndex=-1;
        listFilterIndex=-1;
    }
    


    
    protected void subscribe(int testDataType) {
// subscribe to control server to keep up to date with the list of tests
        listFilterIndex=testDataType;
        try {

         String testServer = SctNames.getTestISServer();
         if(!serverHash.containsKey(testServer)) {
             is.subscribe(testServer, resultObjectPattern, true, this);
//             System.out.println("TestDataInterface: Subscribed to "+testServer+" for objects matching "+resultObjectPattern);
             serverHash.put(testServer, resultObjectPattern);
             }
         String controlServer = SctNames.getControlISServer();
         if(!serverHash.containsKey(controlServer)) {
             is.subscribe(controlServer, controlPattern, true, this);
//             System.out.println("TestDataInterface: Subscribed to "+controlServer+" for objects matching "+controlPattern);
             serverHash.put(controlServer, controlPattern);
             }
         String[] objects = is.getISObjects(controlServer,controlPattern);
         ISTests = new Vector();
// require a the result object to have a matching control object
         for (int j=0; j<objects.length; j++) {  
                  Matcher matcher = objectPattern.matcher(objects[j]);
                  if(!matcher.matches()) continue;
                  String controlObject = objects[j];
                  if(listFilterIndex==-1) ISTests.addElement(controlObject);
                   else {
                       try {
                         TestData testData = new TestData(); 
                         is.getRepository().getValue(controlObject, testData); 
                         String thisTestName = (listFilterIndex==TEST_3PTGAIN) ? testNames[TEST_NPTGAIN] : testNames[listFilterIndex];
                         if(testData.testName.indexOf(thisTestName)!=-1) ISTests.addElement(controlObject);
                       }catch(Exception e2){System.out.println("***ERROR reading "+controlObject+": Test MISSED");}
                       }
                   }  
        } catch (RuntimeException e) {
            System.err.println("Couldn't subscribe to one or more IS Server(s): " + e.getMessage());
        }
//        guiControl.displayPane.setDisplayColors();
//        guiControl.displayPane.refreshDisplay();   

    }

    protected void unsubscribe() {
            for(Enumeration e = serverHash.keys(); e.hasMoreElements();) {
               String server = (String)e.nextElement();
               String pattern = (String)serverHash.get(server);
               is.unsubscribe(server, pattern);
//               System.out.println(is.unsubscribe(server, pattern));
               }
            serverHash = new Hashtable();
            testIndex=-1;
            listFilterIndex=-1;
    }


    
    /** InfoListener interfaces
     */ 
    
    /**
     * Adds the new object - we must run this in the EventDispatch thread to avoid possible thread conflicts
     */
    public void infoCreated(InfoEvent e) {
    // will be invoked if a new control object OR a new test summary object is published
       theObjectName = e.getName();
       javax.swing.SwingUtilities.invokeLater(new Runnable() {
          public void run() {
                  Matcher matcher = objectPattern.matcher(theObjectName);
                  if(matcher.matches()) {
                     // if a new control object, add it to the list
                     if(!ISTests.contains(theObjectName)) ISTests.addElement(theObjectName);
                    }
                  guiControl.tablesDisplayPane.updateTables();  // immediately add the control info to the table
                  guiControl.displayPane.setDisplayColors();
                  guiControl.displayPane.refreshDisplay();  // just refresh the display
          }
        });
    }


    
    /**
     * Object removed from IS -must run this in the EventDispatch thread to avoid possible conflicts
     */
    public void infoDeleted(InfoEvent e) {
       theObjectName = e.getName(); 
       javax.swing.SwingUtilities.invokeLater(new Runnable() {
          public void run() {   
             if(ISTests.contains(theObjectName)) {
                 ISTests.removeElement(theObjectName);
                 }
             guiControl.displayPane.setDisplayColors();
             guiControl.displayPane.refreshDisplay();  // just refresh the display
          }
        });
    }
    
    /**
     * Does nothing - object is still there 
     */
    public void infoUpdated(InfoEvent e) {
        theObjectName = e.getName();
        if(theObjectName.indexOf("ControlData")!=-1) guiControl.tablesDisplayPane.updateTestTable(theObjectName,ISTests.indexOf(theObjectName));
//        System.out.println(e.getName() + " has been updated");
    }
//***************************************************************************
// construct a tabel vector of the current tests
   public Vector getTestTableList() {
       Vector theLine = new Vector();
       Vector tableContent = new Vector();
       theLine.addElement("Test");
       theLine.addElement("Status");
       theLine.addElement("Run");
       theLine.addElement("Scan");
       theLine.addElement("#Scans");
       tableContent.addElement(theLine);
       for(int i=0;i<ISTests.size();i++) {
           TestData testData = new TestData(); 
           is.getRepository().getValue((String)ISTests.elementAt(i), testData); 
           theLine=new Vector();
           if(testData.testName.equals("NPtGainTest") && testData.nScans==3) testData.testName = "3PtGainTest";
           theLine.addElement(testData.testName);
           switch(testData.status) {
             case 0:
                 theLine.addElement("Executing");
                 break;
             case 1:
                 theLine.addElement("Completed");
                 break;
             case 2:
                 theLine.addElement("Aborted");
                 break;
             default:
                 theLine.addElement("Unknown");
             }
//           theLine.addElement(Integer.toString(testData.runNumber));
//           theLine.addElement(Integer.toString(testData.startScanNumber));
//           theLine.addElement(Integer.toString(testData.nScans));
           theLine.addElement(new Integer(testData.runNumber));
           theLine.addElement(new Integer(testData.startScanNumber));
           theLine.addElement(new Integer(testData.nScans));
           tableContent.addElement(theLine);
           }
       return tableContent;
    }

        
//****************************************************************************
     public void setTestFilter(int selectedRow) {
         if(selectedRow==-1) {
             resultObjectHeader=null;
             sctdbObjectHeader=null;
             testObjectHeader=null;
            testIndex=-1;
            return;
            }
         String testObjectName = (String)ISTests.elementAt(selectedRow);
         testData = new TestData(); 
         is.getRepository().getValue(testObjectName, testData); 
// first extract the run number, and scan numbers
         String testName = testData.testName;
         testIndex=-1;
         for(int i=0;i<testNames.length;i++) {
           if(testName.indexOf(testNames[i])!=-1) {
                    if(testName.equals("NPtGainTest") && testData.nScans==3) {
                             testIndex=5;
                             break;
                             }
                    testIndex=i;
                    break;
                    }         
           }

          if(testIndex==-1) {
             System.out.println("sctGui TestDataInterface : Unrecognised testname: "+testName);
             resultObjectHeader=null;
             sctdbObjectHeader=null;
             testObjectHeader=null;
             return;
             }
       
          testObjectHeader = "TestData.SctData::"+testName+"Result."+Integer.toString(testData.runNumber)+"."+Integer.toString(testData.startScanNumber)+".";
          resultObjectHeader = "TestData.Summary."+Integer.toString(testData.runNumber)+"."+Integer.toString(testData.startScanNumber)+".";
          sctdbObjectHeader = "SCTDBData."+testNames[testIndex]+"."+guiControl.sctdbDataInterface.getLocation()+".";
         } 
//***********************************************************************8
     public String getResultObjectHeader() {
          return resultObjectHeader;
          }
//***********************************************************************8
     public String getTestObjectHeader() {
          return testObjectHeader;
          }

//*************************************************************************
     public String getSCTDBObjectHeader() {
          return sctdbObjectHeader;
          }
//**********************************************************************8
     public int getTestIndex() {
         return testIndex;
         }
//*********************************************************************
     public int getListFilterIndex() {
         return listFilterIndex;
         }
   public int getRunNo() {
         if(testData==null) return -1;
         else return testData.runNumber;
         }
   public int getScanNo() {
         if(testData==null) return -1;
         else return testData.startScanNumber;
         }


     private GuiComponents.System.ISInterface is;


}
