package ProdDatabase;
//
// @Author Dave Robinson
//
import java.io.*;
import java.util.Hashtable;
import java.util.Vector;
import Preferences.PreferencesInterface;
import guiUtilities.userPrompt;
import guiUtilities.DaveUtils;

public class SignoffDataInterface implements ModuleSignoffData {

   Hashtable signoffDataHash;
   private String location;
   String signoffType;
   String finalSignoffInfo;

    public SignoffDataInterface(String serialNo, String location, String signoffType) {
            this.signoffType=signoffType;
            try {
              signoffDataHash = GeneralUtilities.getSignoffDataTestNo(serialNo, location, signoffType);
              if(!signoffDataHash.containsKey(location)) return;
              String testno = (String)signoffDataHash.get(location);
              Vector rawData = GeneralUtilities.getRawData(testno);
              if(rawData.size()>0) signoffDataHash.put(location,new SignoffData(GeneralUtilities.getUploaderInfo(testno),(String)rawData.elementAt(1)));
            }catch(Exception e){System.out.println("FAILED to download signoff data: "+e.toString());}
       }
    public SignoffDataInterface(String serialNo, String signoffType) {
            this.signoffType=signoffType;
            try {
              signoffDataHash = GeneralUtilities.getSignoffDataTestNo(serialNo,signoffType);
              // replace test numbers by SignoffData objects
              for (java.util.Enumeration e = signoffDataHash.keys() ; e.hasMoreElements() ;) {
                    String location  = (String)e.nextElement();
                    String testno = (String)signoffDataHash.get(location);
                    Vector rawData = GeneralUtilities.getRawData(testno);
                    if(rawData.size()>0) signoffDataHash.put(location,new SignoffData(GeneralUtilities.getUploaderInfo(testno),(String)rawData.elementAt(1)));
                    else signoffDataHash.remove(location);
                    }
            }catch(Exception e){System.out.println("FAILED to download signoff data: "+e.toString());}
       }

    public String getString(String location,int index, boolean isHTML) {
        if(!signoffDataHash.containsKey(location)) return "None uploaded";
        String value=null;
        if((value=((SignoffData)signoffDataHash.get(location)).getString(index))!=null) {
               if(isHTML) value = value.replaceAll("\\n","<br>");
               switch(index) {
                    case ModuleSignoffData.IVCATEGORY:
                    case ModuleSignoffData.ELECTRICALCATEGORY:
                    case ModuleSignoffData.SCURVECATEGORY:
                      if(value.matches(".*GOOD|Good.*")) value="<font color=green>"+value+"</font>";
                      else value="<font color=red>"+value+"</font>";
                      break;
                    default:
                }
            }
        return (value==null) ? "None uploaded" : value;
        }
//**************************************8
// this method specifically for FINALSIGNOFF data
// where the hash should have only one key-value pair
    public String getString(int index, boolean isHTML) {
        String value=null;
        if(signoffDataHash.size()!=1) return null;
        for (java.util.Enumeration e = signoffDataHash.keys() ; e.hasMoreElements() ;) {
            String location  = (String)e.nextElement();     
            value=((SignoffData)signoffDataHash.get(location)).getString(index);
            if(isHTML) value = value.replaceAll("\\n","<br>");
            }
        return value;
        }
    public Hashtable getSignoffDataHash() {
        return signoffDataHash;
        }
    public void setParameter(String location, int index, String value) {
        if(!signoffDataHash.containsKey(location)) signoffDataHash.put(location, new SignoffData());
        ((SignoffData)signoffDataHash.get(location)).put(keywords[index],value);
        }
    public String getUploaderInfo(String location) {
        if(!signoffDataHash.containsKey(location)) return null;
        return ((SignoffData)signoffDataHash.get(location)).getUploaderInfo();
        }

//**********************************************************************************
    public String writeUploadFiles(String sn, String location) {
        String userInitials=null;
        if(!signoffDataHash.containsKey(location)) return null;
        SignoffData signoffData = (SignoffData)signoffDataHash.get(location);
        try {
          userPrompt thisPrompt = new userPrompt(null,"","Your initials", GeneralUtilities.getNameList(location));
          thisPrompt.pack();
          thisPrompt.setVisible(true);
          userInitials = thisPrompt.getValidatedText();
          if (userInitials.equals("None")) return null;
          } catch(Exception e1){System.out.println("ERROR retrieving list of users from SCTDB: "+e1.toString()); return null;}
        try {
          return writeUploadFile(sn,signoffData.writeUploadDataFile(sn),userInitials, location);
          }catch(Exception e) {System.out.println("FAILED to create upload file: "+e.toString()); return null;}
        }

    private String writeUploadFile(String sn, String dataFile, String userInitials, String location) throws Exception {
        PreferencesInterface prefs = PreferencesInterface.getInstance();
        File outFile = new File(prefs.getPreference(PreferencesInterface.UPLOAD_DIR),sn+"_signoff.upl");
        BufferedWriter out = new BufferedWriter(new FileWriter(outFile));
        out.write("%NEWTEST");
        out.newLine();
        out.write("SERIAL NUMBER : "+sn);
        out.newLine();
        out.write("TEST MADE BY : "+userInitials);
        out.newLine();
        out.write("LOCATION NAME : "+location);
        out.newLine();
        out.write("Run number : "+signoffType);
        out.newLine();
        out.write("TEST DATE : "+DaveUtils.getTodaysDate());
        out.newLine();
        out.write("PASSED : YES");
        out.newLine();
        out.write("PROBLEM : NO");
        out.newLine();
        out.write("%bmMOD_RData");
        out.newLine();
        out.write("%TEST Rawdata");
        out.newLine();
        out.write("FILENAME : "+dataFile);
        // note deliberately miss final newline, otherwise UploadTestData will fail!
        out.close();
        return outFile.getAbsolutePath();
        }
//*****************************************************************************
}