package SummaryReader;
//
// @Author Dave Robinson
//
import java.io.*;
import java.util.Vector;
import java.util.Date;
import java.util.Locale;
import java.util.regex.*;
import java.util.Hashtable;
import java.text.*;

public class Reader implements regExpressions, DisplayGUI.TestDataInfo {


    Matcher matcher;
    Test testObject;
    Vector testObjects;
    int parameterType;
    int testType;
    Hashtable testMapping;


    public Reader(String resultString, int testType, int parameterType) {
            this.testType=testType;
            this.parameterType=parameterType;
            getTestData(resultString);
            }
    public Reader(String resultString) {
            testMapping = getTestMapping();
            getTestInfo(resultString);
            }
    public Reader(String resultString, boolean requestedStripCount, boolean requestedStatus) {
            if(requestedStripCount) getDefectInfo(resultString);
            if(requestedStatus) getTestStatusInfo(resultString);
            }

    public String getData(int index) {
         return testObject.getData(index);
         }
    public String getData() {
         return testObject.getData(-1);
         }
    public Test getTest() {
         return testObject;
         }
   public void getDefectInfo(String resultString) {

      testObject = new Test();

      String[] lines = resultString.split("\\n");      

      for(int i=0;i<lines.length;i++) {
          String line = lines[i];
          matcher = defectSummaryPattern.matcher(line);
          if(matcher.matches()) {
                   testObject.addDefectStripCount(line.substring(matcher.start(2),matcher.end(2))); 
                   break;
                   }  
          if(line.startsWith("#No defects found")) {
                   testObject.addDefectStripCount("0");
                   break;
                   }
          }
      }
   public void getTestStatusInfo(String resultString) {
// buils status flag as a string
// 0 - Pass
// 1 - Problem
// 2 - fail
      testObject = new Test();
      String passFlag=null, problemFlag=null;

      String[] lines = resultString.split("\\n");      

      for(int i=0;i<lines.length;i++) {
          String line = lines[i];
          matcher = passedPattern.matcher(line);
          if(matcher.matches()) {
              passFlag = line.substring(matcher.start(1),matcher.end(1));
              continue;
              }
          matcher = problemPattern.matcher(line);
          if(matcher.matches()) {
              problemFlag = line.substring(matcher.start(1),matcher.end(1));
              }
          if(problemFlag!=null && passFlag!=null) break;
          }
       if(passFlag==null || problemFlag==null) return;
       String statusFlag= (passFlag.equals("YES")) ? "0" : "2";
       if(statusFlag.equals("0.0")) {
          if(problemFlag.equals("YES")) statusFlag="1";
          }
       testObject.addTestStatus(statusFlag);
      }

   public void getTestData(String resultString) {
      boolean[] nextLineIsChipData = {false,false,false,false,false,false,false,false,false,false,false,false};
      boolean loopB=false;
      boolean nextLineIsLink0Data=false;
      boolean nextLineIsLink1Data=false;
      int parameterIndex=-1;
      int byPassSection=-1;
      int byPassChipOffset=-1;

      testObject = new Test();

      String[] lines = resultString.split("\\n");      

      for(int i=0;i<lines.length;i++) {
          String line = lines[i];
          if(line.equals("")) continue; // shouldn't be necessary

// check for new test
             if(line.matches("^%NewTest")) {
                testObject = new Test();
                continue;
             }

             if(testObject==null) continue;

                 matcher = spacerPattern.matcher(line);
                 if(matcher.matches()) line=line.substring(matcher.start(1),matcher.end(1));
                 switch(testType) {
                    case TEST_IV:
                         matcher = iv150Pattern.matcher(line);
                         if(matcher.matches()) {
                               testObject.setData(0,line.substring(matcher.start()+1,matcher.end()));
                               continue;
                               }
                         matcher = iv350Pattern.matcher(line);
                         if(matcher.matches()) {
                               testObject.setData(1,line.substring(matcher.start()+1,matcher.end()));
                               continue;
                               }
                         matcher = iv500Pattern.matcher(line);
                         if(matcher.matches()) {
                               testObject.setData(2,line.substring(matcher.start()+1,matcher.end()));
                               continue;
                               }
                         matcher = temperaturePattern.matcher(line);
                         if(matcher.matches()) {
                              testObject.setData(3,line.substring(matcher.start()+1,matcher.end()));
                              continue;
                              }
                         break;
                    case TEST_PIPELINE:
                         if(nextLineIsLink0Data || nextLineIsLink1Data) {
                           String[] dataLine = line.split("\\s+");
                           if(dataLine.length!=6) break;
                           for(int j=0;j<dataLine.length;j++) {
                                      int index = nextLineIsLink1Data ? j+6 : j;
                                      try {  // subtract 128 from number
                                        int thisData = Integer.parseInt(dataLine[j]);
                                        thisData=128-thisData;
                                        dataLine[j]= (j==3) ? "1" : Integer.toString(thisData);
                                        }catch(Exception e){};
                                      testObject.setData(index,dataLine[j]);
                                      }
                           }
                          nextLineIsLink0Data = line.startsWith("#M0");
                          nextLineIsLink1Data = line.startsWith("#M8");
                          break;
                    case TEST_FULLBYPASS:
//  byPassChipOffset   chip0  chip0   chip1  chip1
//                 0   data0  data1   data0  data1
//                 1   data0  data1   data0  data1
//          ...
// where chip0=m0,s2 etc for byPassChipOffset=0,1 etc
//       chip1=s1,s3 etc for byPassChipOffset=0,1 etc
                         if(line.matches("^#Comment$")) byPassSection=1;
                         if(line.matches("^#token\\s+r_tkn\\s+token\\s+r_tkn$")) byPassSection=0;
                         if(byPassSection==-1) break;
                         if(byPassSection!=0) break;
//                         if(byPassSection==1 && (parameterType==0 || parameterType==1)) break;                          
                         if(byPassChipOffset!=-1) {
                           String[] dataLine = line.split("\\s+");
                           if(dataLine.length!=4) break;
                           int pIndex = (parameterType>1) ? parameterType-2 : parameterType;
                           testObject.setData((byPassChipOffset*2),dataLine[pIndex]);   
                           testObject.setData((byPassChipOffset*2)+1,dataLine[pIndex+2]);                        
                           }
                         byPassChipOffset=-1;
                         for(int offset=0;offset<6;offset++) {
                             if(line.matches(fullBypassChipPattern[offset])) {
                                   byPassChipOffset=offset;
                                   break;
                                   }
                             }
                         break;
                    case TEST_STROBEDELAY:
                         if(nextLineIsLink0Data || nextLineIsLink1Data) {
                           String[] dataLine = line.split("\\s+");
                           if(dataLine.length!=6) break;
                           for(int j=0;j<dataLine.length;j++) {
                                      int index = nextLineIsLink1Data ? j+6 : j;
                                      testObject.setData(index,dataLine[j]);
                                      }
                           }
                          nextLineIsLink0Data = line.startsWith("#M0");
                          nextLineIsLink1Data = line.startsWith("#M8");
                          break;
                    case TEST_3PTGAIN:
                    case TEST_NPTGAIN:
                         if(!loopB) loopB = line.startsWith("#LoopB");
                         if(loopB) {
                           for(int j=0;j<12;j++) {
                              if(nextLineIsChipData[j]) {
                                String[] dataLine = line.split("\\s+");
                                testObject.setData(j,dataLine[parameterType]);
                                }
                              nextLineIsChipData[j] = (line.matches(chipNames[j]));
                              }
                          }
                          break;
                    case TEST_TRIM:
                         for(int j=0;j<12;j++) {
                            if(nextLineIsChipData[j]) {
                                String[] dataLine = line.split("\\s+");
                                if(parameterType==2) {
                                      try {  // subtract 128 from number
                                        int thisData = Integer.parseInt(dataLine[parameterType]);
                                        thisData=128-thisData;
                                        dataLine[parameterType]=Integer.toString(thisData);
                                        }catch(Exception e2){};
                                    }
                                testObject.setData(j,dataLine[parameterType]);
                                }
                             nextLineIsChipData[j] = (line.matches(chipNames[j]));
                             }
                          break;
                    case TEST_NOISE:
                         for(int j=0;j<12;j++) {
                            if(nextLineIsChipData[j]) {
                                String[] dataLine = line.split("\\s+");
                                if(parameterType==1 || parameterType==2) dataLine[parameterType]= (new java.text.DecimalFormat("0.##E0")).format(Double.parseDouble(dataLine[parameterType]));
                                testObject.setData(j,dataLine[parameterType]);
                                }
                             nextLineIsChipData[j] = (line.matches(chipNames[j]));
                             }
                          break;
                    case TEST_TIMEWALK:
                         if(line.startsWith("#TW")) parameterIndex=0;     // 0 = TimeWalk data; 1 = TCal data
                         if(line.startsWith("#TCAL")) parameterIndex=1;
                         if(parameterIndex!=parameterType) break;
                         if(nextLineIsLink0Data || nextLineIsLink1Data) {
                           String[] dataLine = line.split("\\s+");
                           if(dataLine.length!=6) break;
                           for(int j=0;j<dataLine.length;j++) {
                                      int index = nextLineIsLink1Data ? j+6 : j;
                                      testObject.setData(index,dataLine[j]);
                                      }
                           }
                          nextLineIsLink0Data = line.startsWith("#M0");
                          nextLineIsLink1Data = line.startsWith("#M8");
                          break;
                    default:
                    }            

          }
   }
//***************************************************************************8
   public void getTestInfo(String resultString) {
      boolean[] nextLineIsChipData = {false,false,false,false,false,false,false,false,false,false,false,false};
      boolean loopB=false;
      boolean nextLineIsLink0Data=false;
      boolean nextLineIsLink1Data=false;
      boolean nextLineIsVersion=false;
      boolean nextLineIsTime=false;
      boolean nextLineIsHost=false;
      boolean nextLineIsNoScans=false;
      boolean nextLineIsScanType=false;
      boolean nextLineIsScanPoints=false;
      boolean nextLineIsComment=false;
      int scanCount=0;
      int parameterIndex=-1;
      int byPassSection=-1;
      int byPassChipOffset=-1;

      testObject = new Test();
      DataBlock dataBlock = new DataBlock("",1);
      DAQInfo daqInfo = new DAQInfo();
      TestInfo testInfo = new TestInfo();
      ScanInfo scanInfo = new ScanInfo();
      DefectInfo defectInfo = new DefectInfo();
      int testno=999;

      String[] lines = resultString.split("\\n");      

      for(int i=0;i<lines.length;i++) {
          String line = lines[i];
          if(line.equals("")) continue; // shouldn't be necessary
//          System.out.println(line);

// check for new test
             if(line.matches("^%NewTest")) {
                testObject = new Test();
                continue;
             }

             if(testObject==null) continue;

// first the serial No
             matcher = serialNoPattern.matcher(line);
             if(matcher.matches()) {
                  String thisSerialNo = line.substring(matcher.start(1),matcher.end(1));
                  testObject.setSerialNo(thisSerialNo);
                  continue;
                  }

// then the run number and scan number
                 matcher = runPattern.matcher(line);
                 if(matcher.matches()) {
                           String runString = line.substring(matcher.start(1),matcher.end(1));
                           matcher = scanNoPattern.matcher(runString);
                           if(matcher.matches()) {
                                 testInfo.setRunNo(runString.substring(matcher.start(1),matcher.end(1)));
                                 testInfo.setScanNo(runString.substring(matcher.start(2),matcher.end(2)));
                                 }
                           else {
                                 testInfo.setRunNo(runString);
                                 testInfo.setScanNo("None");
                                 }
                           continue;
                           }
// then the date
                 matcher = testDatePattern.matcher(line);
                 if(matcher.matches()) {
                     daqInfo.setDate(line.substring(matcher.start(1),matcher.end(1)));
                     continue;
                    }
// now passed or fail status
                 matcher = passedPattern.matcher(line);
                 if(matcher.matches()) {
                     testInfo.setPassed(line.substring(matcher.start(1),matcher.end(1)));
                     continue;
                     }
                 matcher = problemPattern.matcher(line);
                 if(matcher.matches()) {
                     testInfo.setProblem(line.substring(matcher.start(1),matcher.end(1)));
                     continue;
                     }
// now the test type
                 matcher = testregex.matcher(line);
                 if(matcher.matches()) {
                        String theTestType = line.substring(matcher.start()+1,matcher.end());
                        testInfo.setName(theTestType);
                        if(testMapping.containsKey(theTestType)) testno = ((Integer)testMapping.get(theTestType)).intValue();
                        switch(testno) {
                          case TEST_PIPELINE:
                            dataBlock = new DataBlock("Number of Good Channels",1);
                            dataBlock.add(0,"#Channels");
                            break;
                          case TEST_FULLBYPASS:                          
                             dataBlock = new DataBlock("FullBypass Data",testParameters[testno].length-2);
                             for(int header=2;header<testParameters[testno].length;header++) dataBlock.add(header-2,testParameters[testno][header]);
                             break;
                          case TEST_STROBEDELAY: 
                            dataBlock = new DataBlock("Optimum Strobe Delays",1);
                            dataBlock.add(0,"Delay");
                            break;
                          case TEST_TRIM:                          
                             dataBlock = new DataBlock("Trim Data",testParameters[testno].length-2);
                             for(int header=2;header<testParameters[testno].length;header++) dataBlock.add(header-2,testParameters[testno][header]);
                             break;
                          case TEST_NOISE:
                             dataBlock = new DataBlock("Noise Data",testParameters[testno].length-2);
                             for(int header=2;header<testParameters[testno].length;header++) dataBlock.add(header-2,testParameters[testno][header]);
                             break;
                          case TEST_TIMEWALK:
                            dataBlock = new DataBlock("TimeWalk Data",testParameters[testno].length-2);
                             for(int header=2;header<testParameters[testno].length;header++) dataBlock.add(header-2,testParameters[testno][header]);
                             break;
                          default:
                          }
                        continue;
                        }
//
                 matcher = spacerPattern.matcher(line);
                 if(matcher.matches()) line=line.substring(matcher.start(1),matcher.end(1));
                 switch(testno) {
                    case TEST_IV:
                         matcher = iv150Pattern.matcher(line);
                         if(matcher.matches()) {
                               testObject.setData(0,line.substring(matcher.start()+1,matcher.end()));
                               continue;
                               }
                         matcher = iv350Pattern.matcher(line);
                         if(matcher.matches()) {
                               testObject.setData(1,line.substring(matcher.start()+1,matcher.end()));
                               continue;
                               }
                         matcher = iv500Pattern.matcher(line);
                         if(matcher.matches()) {
                               testObject.setData(2,line.substring(matcher.start()+1,matcher.end()));
                               continue;
                               }
                         matcher = temperaturePattern.matcher(line);
                         if(matcher.matches()) {
                              testObject.setData(3,line.substring(matcher.start()+1,matcher.end()));
                              continue;
                              }
                         break;
                    case TEST_PIPELINE:
                         if(nextLineIsLink0Data || nextLineIsLink1Data) {
                           String[] dataLine = line.split("\\s+");
                           if(dataLine.length!=6) break;
                           for(int j=0;j<dataLine.length;j++) {
                                      int index = nextLineIsLink1Data ? j+6 : j;
                                      dataBlock.add(index,0,dataLine[j]);
                                      }
                           }
                          nextLineIsLink0Data = line.startsWith("#M0");
                          nextLineIsLink1Data = line.startsWith("#M8");
                          break;
                    case TEST_FULLBYPASS:
//  byPassChipOffset   chip0  chip0   chip1  chip1
//                 0   data0  data1   data0  data1
//                 1   data0  data1   data0  data1
//          ...
// where chip0=m0,s2 etc for byPassChipOffset=0,1 etc
//       chip1=s1,s3 etc for byPassChipOffset=0,1 etc
                         if(line.matches("^#Comment$")) byPassSection=1;
                         if(line.matches("^#token\\s+r_tkn\\s+token\\s+r_tkn$")) byPassSection=0;
                         if(byPassSection==-1) break;
                         if(byPassSection!=0) break;
//                         if(byPassSection==1 && (parameterType==0 || parameterType==1)) break;                          
                         if(byPassChipOffset!=-1) {
                           String[] dataLine = line.split("\\s+");
                           if(dataLine.length!=4) break;
                           int pIndex = (parameterType>1) ? parameterType-2 : parameterType;
                           dataBlock.add((byPassChipOffset*2),parameterType,dataLine[pIndex]);   
                           dataBlock.add((byPassChipOffset*2)+1,parameterType,dataLine[pIndex+2]);                        
                           }
                         byPassChipOffset=-1;
                         for(int offset=0;offset<6;offset++) {
                             if(line.matches(fullBypassChipPattern[offset])) {
                                   byPassChipOffset=offset;
                                   break;
                                   }
                             }
                         break;

                    case TEST_STROBEDELAY:
                         if(nextLineIsLink0Data || nextLineIsLink1Data) {
                           String[] dataLine = line.split("\\s+");
                           if(dataLine.length!=6) break;
                           for(int j=0;j<dataLine.length;j++) {
                                      int index = nextLineIsLink1Data ? j+6 : j;
                                      dataBlock.add(index,0,dataLine[j]);
                                      }
                           }
                          nextLineIsLink0Data = line.startsWith("#M0");
                          nextLineIsLink1Data = line.startsWith("#M8");
                          break;
                    case TEST_3PTGAIN:
                    case TEST_NPTGAIN:
                         if(line.startsWith("#LoopA")) {
                             dataBlock = new DataBlock("Fit Parameters",4);
                             dataBlock.add(0,"Function");
                             dataBlock.add(1,"P0");
                             dataBlock.add(2,"P1");
                             dataBlock.add(3,"P2");         
                             continue;
                             }
                         if(line.startsWith("#LoopB")) {
                             if(dataBlock!=null) testObject.add(dataBlock);
                             dataBlock = new DataBlock("NPtGain Data",testParameters[testno].length-2);
                             for(int header=2;header<testParameters[testno].length;header++) dataBlock.add(header-2,testParameters[testno][header]);
                             continue;
                             }
                         for(int j=0;j<12;j++) {
                              if(nextLineIsChipData[j]) {
                                String[] dataLine = line.split("\\s+");
                                for(int k=0;k<dataLine.length;k++) {
                                         dataBlock.add(j,k,dataLine[k]);
                                         }
                                }
                              nextLineIsChipData[j] = (line.matches(chipNames[j]));
                          }
                          break;
                    case TEST_TRIM:
                         for(int j=0;j<12;j++) {
                            if(nextLineIsChipData[j]) {
                                String[] dataLine = line.split("\\s+");
                                for(int k=0;k<dataLine.length;k++) {
                                      if(k==2) {
                                         try {  // subtract 128 from number
                                          int thisData = Integer.parseInt(dataLine[parameterType]);
                                          thisData=128-thisData;
                                          dataLine[parameterType]=Integer.toString(thisData);
                                          }catch(Exception e2){};
                                       }
                                     dataBlock.add(j,k,dataLine[k]);
                                     }
                                }
                             nextLineIsChipData[j] = (line.matches(chipNames[j]));
                             }
                          break;
                    case TEST_NOISE:
                          for(int j=0;j<12;j++) {
                            if(nextLineIsChipData[j]) {
                                String[] dataLine = line.split("\\s+");
                                for(int k=0;k<dataLine.length;k++) {
                                        if(k==1 || k==2) dataLine[k]= (new java.text.DecimalFormat("0.##E0")).format(Double.parseDouble(dataLine[k]));
                                        dataBlock.add(j,k,dataLine[k]);
                                        }
                                }
                             nextLineIsChipData[j] = (line.matches(chipNames[j]));
                             }
                          break;
                    case TEST_TIMEWALK:
                         if(line.startsWith("#TW")) parameterIndex=0;     // 0 = TimeWalk data; 1 = TCal data
                         if(line.startsWith("#TCAL")) parameterIndex=1;
                         if(parameterIndex!=parameterType) break;
                         if(nextLineIsLink0Data || nextLineIsLink1Data) {
                           String[] dataLine = line.split("\\s+");
                           if(dataLine.length!=6) break;
                           for(int j=0;j<dataLine.length;j++) {
                                      int index = nextLineIsLink1Data ? j+6 : j;
                                      dataBlock.add(index,parameterIndex,dataLine[j]);
                                      }
                           }
                          nextLineIsLink0Data = line.startsWith("#M0");
                          nextLineIsLink1Data = line.startsWith("#M8");
                          break;
                    default:
                    }     
          if(nextLineIsScanPoints) {
             if(line.startsWith("%") || line.startsWith("#")) {
                  nextLineIsScanPoints=false;
                  continue;
                  }
             try {
                String[] scanPoints = line.split("\\s+");
                for(int k=0;k<scanPoints.length;k++) {
                       scanInfo.setScanPoint(scanCount+k,scanPoints[k]);
                       }
                 scanCount+=scanPoints.length;
                 continue;
             }catch(Exception e3){}
          }

          matcher = defectNamePattern.matcher(line);
          if(matcher.matches()) {
             defectInfo = new DefectInfo();
             defectInfo.setName(line.substring(matcher.start(1),matcher.end(1)));
             continue;
             }
          matcher = defectNameDefect0.matcher(line);
          if(matcher.matches()) {
             defectInfo.setFirstChannel(line.substring(matcher.start(1),matcher.end(1)));
             continue;
             }
          matcher = defectNameDefect1.matcher(line);
          if(matcher.matches()) {
             defectInfo.setLastChannel(line.substring(matcher.start(1),matcher.end(1)));
             testObject.add(defectInfo);
             continue;
             }

         if(nextLineIsComment) {
             matcher = commentsPattern.matcher(line);
             if(matcher.matches()) testObject.addComment(line.substring(matcher.start(1),matcher.end(1)));
             }

             
          if(nextLineIsVersion) daqInfo.setVersion(line);
          if(nextLineIsTime) daqInfo.setTime(line.replaceAll("\"",""));
          if(nextLineIsHost) daqInfo.setHost(line.replaceAll("\"",""));
          if(nextLineIsScanType) scanInfo.setType(line);
          if(nextLineIsNoScans) scanInfo.setPoints(line);
 
          nextLineIsVersion = line.matches("^[\\s#]?VERSION.*");
          nextLineIsTime = line.matches("^#TIME$");
          nextLineIsHost = line.matches("^#HOST$");      
          nextLineIsScanType = line.matches("^#POINT_TYPE$");
          nextLineIsNoScans = line.matches("^#N_POINTS$");    
          nextLineIsScanPoints = line.matches("^#POINTS$");
          nextLineIsComment = line.matches("^%Comment$");

          }
       testObject.add(dataBlock);
       testObject.add(testInfo);
       testObject.add(daqInfo);
       testObject.add(scanInfo);
   }

  private Hashtable getTestMapping() {
      Hashtable mapping = new Hashtable();
      mapping.put("StrobeDelay",new Integer(TEST_STROBEDELAY));
      mapping.put("ThreePointGain",new Integer(TEST_3PTGAIN));
      mapping.put("ResponseCurve",new Integer(TEST_NPTGAIN));
      mapping.put("PipelineTest",new Integer(TEST_PIPELINE));
      mapping.put("NO",new Integer(TEST_NOISE));
      mapping.put("Trim",new Integer(TEST_TRIM));
      mapping.put("DetModIV",new Integer(TEST_IV));
      return mapping;
      }


     



}
